/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : AddFile.cpp
 * DATE   : 6.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "AddFile.h"
#include "Shared.h"
#include <QGroupBox>
#include <QCheckBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QLayout>
#include <QFileInfo>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const AddFile::CAPTION      = QT_TR_NOOP( "Add file to project" );
const char* const AddFile::GBOX_CAPTION = QT_TR_NOOP( "Add this file types" );
const char* const AddFile::SOURCE_FILE  = QT_TR_NOOP( "Source file (cpp)" );
const char* const AddFile::HEADER_FILE  = QT_TR_NOOP( "Header file (h)" );
const char* const AddFile::FILTER       = QT_TR_NOOP( "Source files (*.cpp *.cc *.cxx);;Header files (*.h *.hpp *.hxx);;Form files (*.ui);;Resource files (*.qrc)" );

//*******************************************************************
// AddFile                                               CONSTRUCTOR
//*******************************************************************
AddFile::AddFile( QWidget * const in_parent, const QString& in_dir )
: QFileDialog     ( in_parent )
, d_box           ( new QGroupBox( tr( GBOX_CAPTION ), this ) )
, d_box_layout    ( new QHBoxLayout( d_box ) )
, d_source        ( new QCheckBox( tr( SOURCE_FILE ), d_box ) )
, d_header        ( new QCheckBox( tr( HEADER_FILE ), d_box ) )
, d_selected_file ( "" )
{
   d_box_layout->addWidget( d_source );
   d_box_layout->addWidget( d_header );

   layout()->addWidget( new QLabel( this ) );
   layout()->addWidget( d_box );

   setWindowTitle( tr( CAPTION ) );
   setDirectory( in_dir );
   setFileMode( AnyFile );
   setFilters( tr( FILTER ).split( ";;" ) );

   connect( this, SIGNAL( currentChanged( const QString& ) ), this, SLOT( current_changed( const QString& ) ));
   connect( this, SIGNAL( accepted() ), this, SLOT( accept() ));
}
// end of AddFile

//*******************************************************************
// current_changed                                      PRIVATE slot
//*******************************************************************
void AddFile::current_changed( const QString& in_fpath /* ,cps*/)
{
   const QString fext = Shared::extension( in_fpath );
   const bool    stat = ( Shared::is_src_ext( fext ) || Shared::is_hdr_ext( fext ) );

   d_source->setChecked( stat );
   d_header->setChecked( stat );
}
// end of current_changed

//*******************************************************************
// accept                                               PRIVATE slot
//*******************************************************************
void AddFile::accept()
{
   const QStringList files( selectedFiles() );
   if( !files.isEmpty() ) {
      const QFileInfo finfo( files.at( 0 ).trimmed() );
      if( !finfo.fileName().isEmpty() ) {
         const QString fext = Shared::extension( finfo.fileName() );
         const bool stat = fext.isEmpty()             ||
                           Shared::is_src_ext( fext ) ||
                           Shared::is_hdr_ext( fext ) ||
                           Shared::is_ui_ext ( fext ) ||
                           Shared::is_qrc_ext( fext );
         if( stat ) {
            d_selected_file = finfo.absoluteFilePath();
            done( 2 );
         }
      }
   }
}
// end of accept

//*******************************************************************
// source_is_checked                                          PUBLIC
//*******************************************************************
bool AddFile::source_is_checked() const
{
   return d_source->isChecked();
}
// end of source_is_checked

//*******************************************************************
// header_is_checked                                          PUBLIC
//*******************************************************************
bool AddFile::header_is_checked() const
{
   return d_header->isChecked();
}
// end of header_is_checked

//*******************************************************************
// selected_file                                              PUBLIC
//*******************************************************************
const QString& AddFile::selected_file() const
{
   return d_selected_file;
}
// end of header_is_checked
