/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : BookmarksList.h
 * DATE   : 11.11.2006
 *******************************************************************/
#ifndef INCLUDED_BookmarksList_h
#define INCLUDED_BookmarksList_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_QDialog
#define INCLUDED_QDialog
#include <QDialog>
#endif // INCLUDED_QDialog

#ifndef INCLUDED_Shared_h
#include "Shared.h"
#endif // INCLUDED_Shared_h


/*------- forward declarations:
-------------------------------------------------------------------*/
class QHBoxLayout;
class QVBoxLayout;
class QGroupBox;
class QListWidget;
class QPushButton;
class QListWidgetItem;

/*------- class declaration:
-------------------------------------------------------------------*/
class BookmarksList : public QDialog
{
	Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
	explicit BookmarksList( QWidget* );
private:
	BookmarksList( const BookmarksList& );
	BookmarksList& operator=( const BookmarksList& );

// ******* TYPES *******
private:
	enum {
		PATH_ROLE = Qt::UserRole + 1,
		NUMBER_ROLE,
	};

// ******* CONSTANTS *******
private:
	static const char* const CAPTION;
	static const char* const FILES;
	static const char* const BOOKMARKS;
	static const char* const CLEAR_LABEL;
	static const char* const GO_LABEL;
	static const char* const EXIT_LABEL;

// ******* MEMBERS *******
private:
	QHBoxLayout*  const d_mainlay;
	QGroupBox*    const d_files_gbox;
	QVBoxLayout*  const d_files_lay;
	QListWidget*  const d_files_lstw;
	QGroupBox*    const d_bookmarks_gbox;
	QVBoxLayout*  const d_bookmarks_lay;
	QListWidget*  const d_bookmarks_lstw;
	QVBoxLayout*  const d_btn_lay;
	QPushButton*  const d_clear_btn;
	QPushButton*  const d_go_btn;
	QPushButton*  const d_exit_btn;
	QString             d_selected_file;
	qint32              d_selected_line;

// ******* METHODS *******
public:
	void set_bookmarks_info    ( const QVector<FileBookmarks>& );
	bool get_selected_bookmark ( QString&, qint32& ) const;
private:
	void          showEvent ( QShowEvent* );
private slots:
	void       file_changed ( QListWidgetItem* );
	void          clear_all ();
	void  bookmark_selected ( QListWidgetItem* );
	void      goto_bookmark ();
};

inline bool BookmarksList::get_selected_bookmark( QString& out_file, qint32& out_line ) const {
	out_file = d_selected_file;
	out_line = d_selected_line;
	return ( ( false == d_selected_file.isEmpty() ) && ( d_selected_line != -1 ) );
}
/*
inline void BookmarksList::set_bookmarks_info( const QVector<FileBookmarks>& in_bookmarks ) {
	d_bookmarks = in_bookmarks;
}
*/
#endif // INCLUDED_BookmarksList_h
