/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : BrowserSearch.cpp
 * DATE   : 24.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "BrowserMessages.h"
#include "Config.h"
#include "EventsController.h"
#include <QFileInfo>
#include <QMouseEvent>
#include <QtDebug>


//*******************************************************************
// BrowserMessages                                       CONSTRUCTOR
//*******************************************************************
BrowserMessages::BrowserMessages( QWidget* const in_parent )
: QListWidget  ( in_parent )
, d_dir        ( "" )
, d_org_palette( palette() )
, d_org_font   ( font() )
{
   set_properties();

   EventsController::instance()->append( this, Event::PROCESS_STARTED );
   EventsController::instance()->append( this, Event::PROCESS_MESSAGES_DATA );
   EventsController::instance()->append( this, Event::MSG_TAB_PROP_CHANGED );
   
   connect( this, SIGNAL( itemActivated( QListWidgetItem* ) ), this, SLOT( selection( QListWidgetItem* ) ) );
}
// end of BrowserSearch

//*******************************************************************
// ~BrowserMessages                                       DESTRUCTOR
//*******************************************************************
BrowserMessages::~BrowserMessages()
{
   EventsController::instance()->remove( this );
}
//end of ~BrowserMessages

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void BrowserMessages::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<int>( event->type() );

   switch( type ) {
      case Event::PROCESS_STARTED:
         clear();
         d_dir = event->d_data.at( 0 ).toString();
         break;
      case Event::PROCESS_MESSAGES_DATA:
         add_item( event );
         break;
      case Event::MSG_TAB_PROP_CHANGED:
         set_properties();
         break;
   }
}
// end of customEvent

//*******************************************************************
// set_properties                                            PRIVATE
//*******************************************************************
void BrowserMessages::set_properties()
{
   setAlternatingRowColors( Config::instance()->msg_tab_use_altbkg() );
   
   if( Config::instance()->msg_tab_use_defaults() ) {
      setPalette( d_org_palette );
      setFont( d_org_font );
   }
   else {
      QPalette p( d_org_palette );
      p.setColor( QPalette::Text, QColor::fromRgba( Config::instance()->msg_tab_font_color() ) );
      p.setColor( QPalette::Base, QColor::fromRgba( Config::instance()->msg_tab_base_bkg() ) );
      p.setColor( QPalette::AlternateBase, QColor::fromRgba( Config::instance()->msg_tab_altbase_bkg() ) );
      setPalette( p );
   
      // Ustawienie nowego fontu.
      QFont fnt;
      fnt.fromString( Config::instance()->msg_tab_font() );
      setFont( fnt );
   }
}
// end set_properties

//*******************************************************************
// selection                                            PRIVATE slot
//*******************************************************************
void BrowserMessages::selection( QListWidgetItem* const in_item )
{
   if( in_item ) {
      QString text_of_item = in_item->text();
      remove_marker( text_of_item );
      
      const QStringList tokens = text_of_item.split( ':', QString::SkipEmptyParts );
      if( tokens.size() >= 2 ) {
         const QFileInfo fi( d_dir + tokens[0] );
         EventsController::instance()->send_event( Event::SELECTED_FILE_AND_ROW,
                                                   fi.absoluteFilePath(),
                                                   tokens[ 1 ].toInt() );
      }
   }
}
// end of mouseDoubleClickEvent

//*******************************************************************
// add_item                                                  PRIVATE
//*******************************************************************
void BrowserMessages::add_item( Event* const in_event )
{
   const QStringList lines = in_event->d_data.at( 0 ).toString().split( '\n', QString::SkipEmptyParts );
   if( !lines.isEmpty() ) {
      foreach( QString txt, lines ) {
         addItem( txt.trimmed() );
      }
   }
}
// end of add_item

//*******************************************************************
// remove_marker                                             PRIVATE
//*******************************************************************
void BrowserMessages::remove_marker( QString& inout_text ) const
{
   const QString marker = "[>] ";
   
   if( inout_text.startsWith( marker ) ) {
      inout_text = inout_text.remove( 0, marker.length() );
   }
   inout_text = inout_text.trimmed();
}
// end of remove_marker
