#ifndef INCLUDED_CBtCodeBlockInfo_h
#define INCLUDED_CBtCodeBlockInfo_h
/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : CBtCodeBlockInfo.cpp
 * DATE   : 2.08.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QObject>

/*------- class declaration:
-------------------------------------------------------------------*/
class CBtCodeBlockInfo : public QObject
{
//******* CONSTRUCTION / DESTRUCTION *******
public:
   CBtCodeBlockInfo() : QObject()
   , pos_idx_   ( -1 )
   , open_idx_  ( -1 )
   , close_idx_ ( -1 )
   , header_    ( QString() )
   {}
   CBtCodeBlockInfo( const CBtCodeBlockInfo& rhs )
   : QObject    ()
   , pos_idx_   ( rhs.pos_idx_    )
   , open_idx_  ( rhs.open_idx_   )
   , close_idx_ ( rhs.close_idx_  )
   , header_    ( rhs.header_     )
   {}
   CBtCodeBlockInfo& operator=( const CBtCodeBlockInfo& rhs ) {
      if( this != &rhs ) {
         pos_idx_   = rhs.pos_idx_;
         open_idx_  = rhs.open_idx_;
         close_idx_ = rhs.close_idx_;
         header_    = rhs.header_;
      }
      return *this;
   }
   void clear() {
      pos_idx_ = open_idx_ = close_idx_  = -1;
      header_.clear();
   }
   bool valid() const {
      if( -1 == close_idx_ ) return false;
      if( -1 == open_idx_  ) return false;
      if( -1 == pos_idx_   ) return false;
      return true;
   }

//******* MEMBERS *******
private:
   int     pos_idx_;
   int     open_idx_;
   int     close_idx_;
   QString header_;

//******* METHODS *******
public:
   // setters
   void set_pos_idx   ( const int      in_idx     ) { pos_idx_   = in_idx;    }
   void set_open_idx  ( const int      in_idx     ) { open_idx_  = in_idx;    }
   void set_close_idx ( const int      in_idx     ) { close_idx_ = in_idx;    }
   void set_header    ( const QString& in_header  ) { header_    = in_header; }
   // getters
   int            pos_idx   () const { return pos_idx_;    }
   int            open_idx  () const { return open_idx_;   }
   int            close_idx () const { return close_idx_;  }
   const QString& header    () const { return header_;     }
};

#endif // INCLUDED_CBtCodeBlockInfo_h
