#ifndef INCLUDED_CBtUserData_h
#define INCLUDED_CBtUserData_h
/********************************************************************
 * Copyright (C) 2006-2008 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : CBtUserData.h
 * DATE   : 2.07.2008
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Cobras.h"
#include "CBtCodeBlockInfo.h"
#include <QTextBlockUserData>


/*------- class declaration:
-------------------------------------------------------------------*/
class CBtUserData : public QTextBlockUserData
{
//******* CONSTRUCTION / DESTRUCTION *******
public:
   CBtUserData() : QTextBlockUserData()
   , row_        ( -1 )
   , bookmark_   ( false )
   , breakpoint_ ( false )
   , folding_    ( FOLDER_NO_MARKER )
   , code_       ( CBtCodeBlockInfo() )
   {}
private:
   CBtUserData( const CBtUserData& );
   CBtUserData& operator=( const CBtUserData& );

//******* MEMBERS *******
private:
   int              row_;
   bool             bookmark_;
   bool             breakpoint_;
   int              folding_;
   CBtCodeBlockInfo code_;

//******* METHODS *******
public:
   // geters -------
   int row() const {
      return row_;
   }
   bool bookmark() const {
      return bookmark_;
   }
   bool breakpoint() const {
      return breakpoint_;
   }
   int folding() const {
      return folding_;
   }
   const CBtCodeBlockInfo& code() const {
      return code_;
   }

   // seters -------
   void row( const int in_row ) {
      row_ = in_row;
   }
   void bookmark( const bool in_bookmark ) {
      bookmark_ = in_bookmark;
   }
   void breakpoint( const bool in_breakpoint ) {
      breakpoint_ = in_breakpoint;
   }
   void folding( const int in_folding ) {
      folding_ = in_folding;
   }
   void code( const CBtCodeBlockInfo& in_code ) {
      code_ = in_code;
   }
};

#endif // INCLUDED_CBtUserData
