/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Config.cpp
 * DATE   : 24.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Config.h"
#include "Settings.h"
#include <QApplication>
#include <QtDebug>

/*------- local variables:
-------------------------------------------------------------------*/
Config* Config::d_instance = 0;

const bool        Config::DEFAULT_PROJECT_LOAD        = false;
const bool        Config::DEFAULT_FILES_LOAD          = false;
const char* const Config::DEFAULT_QMAKE_PATH          = "";
const bool        Config::DEFAULT_PARSE_CLASS_ONSAVE  = true;
const char* const Config::DEFAULT_EDITOR_TEXT_FONT    = "Monospace,10,-1,5,50,0,0,0,0,0";
const bool        Config::DEFAULT_HIGHLIGHT_LINE      = true;
const qint32      Config::DEFAULT_TAB_WIDTH           = 4;
const bool        Config::DEFAULT_AUTO_INDENT         = true;
const bool        Config::DEFAULT_SYNTAX_HIGHLIGHT    = true;
const QRgb        Config::DEFAULT_CPP_BACKGROUND      = qRgb( 255, 255, 255 );
const QRgb        Config::DEFAULT_HPP_BACKGROUND      = qRgb( 255, 255, 255 );
const QRgb        Config::DEFAULT_CURRLINE_BACKGROUND = qRgb( 240, 240, 220 );
const QRgb        Config::DEFAULT_TEXT_COLOR          = qRgb(   0,   0,   0 );
const QRgb        Config::DEFAULT_COMMENT_COLOR       = qRgb( 115, 111, 224 );
const QRgb        Config::DEFAULT_STRING_COLOR        = qRgb( 255,   0, 255 );
const QRgb        Config::DEFAULT_CHAR_COLOR          = qRgb( 255,   0, 255 );
const QRgb        Config::DEFAULT_DECNUM_COLOR        = qRgb(   0,   0, 255 );
const QRgb        Config::DEFAULT_HEXNUM_COLOR        = qRgb( 115, 111, 224 );
const QRgb        Config::DEFAULT_KEYWORD_COLOR       = qRgb( 165,  42,  42 );
const QRgb        Config::DEFAULT_TYPE_COLOR          = qRgb(  46, 139,  87 );
const QRgb        Config::DEFAULT_PRAGMA_COLOR        = qRgb( 160,  32, 240 );
const QRgb        Config::DEFAULT_CLASS_COLOR         = qRgb( 172,  95, 105 );
const QRgb        Config::DEFAULT_STL_COLOR           = qRgb( 170, 149, 105 );
const QRgb        Config::DEFAULT_QT_COLOR            = qRgb(  46, 139,  87 );
// FileTree, ClassTree, SearchTab
const bool        Config::DEFAULT_USE_DEFAULTS = true;
const bool        Config::DEFAULT_USE_ALTBKG   = false;
const QRgb        Config::DEFAULT_BASE_BKG     = qRgb( 255, 255, 255 );
const QRgb        Config::DEFAULT_ALTBASE_BKG  = qRgb( 246, 244, 241 );


// Goto Line
const QString     Config::GOTO_LINE_GROUP             = "GotoLine/";
const char* const Config::GOTO_LINE_KEY               = "Data";
// Find - Replace
const QString     Config::FIND_GROUP                  = "Find/";
const char* const Config::FIND_KEY                    = "Find";
const char* const Config::REPLACE_KEY                 = "Replace";
const char* const Config::CONFIG_KEY                  = "Config";
// File Tree, Class Tree, MsgTab, SearchTab
const QString     Config::FLTREE_GROUP                = "FileTree/";
const QString     Config::CLTREE_GROUP                = "ClassTree/";
const QString     Config::BUILD_TAB_GROUP             = "BuildTab/";
const QString     Config::SEARCH_TAB_GROUP            = "SearchTab/";
const QString     Config::MSG_TAB_GROUP               = "MsgTab/";
const char* const Config::USE_DEFAULTS_KEY            = "UseDefaults";
const char* const Config::USE_ALTBKG_KEY              = "UseAltBkg";
const char* const Config::BASE_BKG_KEY                = "BaseBkg";
const char* const Config::ALTBASE_BKG_KEY             = "AltBaseBkg";
const char* const Config::FONT_KEY                    = "Font";
const char* const Config::FONT_COLOR_KEY              = "FontColor";


const QString     Config::MAIN_WINDOW_GROUP           = "MainWindow/";
const QString     Config::PROPERTIES_GROUP            = "Properties/";
const char* const Config::LOAD_PROJECT_KEY            = "LoadProjectAtStart";
const char* const Config::LOAD_FILES_KEY              = "LoadFilesAtStart";
const char* const Config::QT_MAKE_PATH_KEY            = "QmakeLocation";
const char* const Config::BEFORE_QT_MAKE_KEY          = "QmakeBefore";
const char* const Config::AFTER_QT_MAKE_KEY           = "QmakeAfter";
const char* const Config::PARSE_CLASS_KEY             = "ParseClassOnSave";
const char* const Config::MENU_FILES_KEY              = "MenuFiles";
const char* const Config::MENU_PROJECTS_KEY           = "MenuProject";
const char* const Config::LAST_OPENED_WINDOWS_KEY1    = "LastOpenedWindows1";
const char* const Config::LAST_OPENED_WINDOWS_KEY2    = "LastOpenedWindows2";
const char* const Config::EDITOR_DEFAULT_KEY          = "UseDefaults";
const char* const Config::EDITOR_HIGHLIGHT_LINE_KEY   = "HighlightCurrentLine";
const char* const Config::EDITOR_TAB_WIDTH_KEY        = "TabWidth";
const char* const Config::EDITOR_AUTO_INDENT_KEY      = "AutoIndentation";
const char* const Config::EDITOR_SYNTAX_HIGHLIGHT_KEY = "SyntaxHighlighting";
const char* const Config::EDITOR_TEXT_PROP_KEY        = "Text";
const char* const Config::EDITOR_COMMENT_PROP_KEY     = "Comment";
const char* const Config::EDITOR_STRING_PROP_KEY      = "String";
const char* const Config::EDITOR_CHAR_PROP_KEY        = "Char";
const char* const Config::EDITOR_DECNUM_PROP_KEY      = "DecNum";
const char* const Config::EDITOR_HEXNUM_PROP_KEY      = "HexNum";
const char* const Config::EDITOR_KEYWORD_PROP_KEY     = "Keyword";
const char* const Config::EDITOR_TYPE_PROP_KEY        = "Type";
const char* const Config::EDITOR_PRAGMA_PROP_KEY      = "Pragma";
const char* const Config::EDITOR_CLASS_PROP_KEY       = "Class";
const char* const Config::EDITOR_QT_PROP_KEY          = "Qt";
const char* const Config::EDITOR_STL_PROP_KEY         = "Stl";
const char* const Config::EDITOR_TAB2SPC_KEY          = "Tab2Spc";
const char* const Config::CPP_BACKGROUND_KEY          = "CppBackground";
const char* const Config::HPP_BACKGROUND_KEY          = "HppBackground";
const char* const Config::CURRLINE_BACKGROUND_KEY     = "CurrLineBackground";
// ******* MARGINS **************************************************
const QString     Config::MARGIN_GROUP                = "Margins";
const char* const Config::MARGIN_USE_NUMBERS_KEY      = "UseNumbers";
const char* const Config::MARGIN_USE_DEFAULTS_KEY     = "UseDefaults";
const char* const Config::MARGIN_FONT_COLOR_KEY       = "FontColor";
const char* const Config::MARGIN_BKG_COLOR_KEY        = "BackgroundColor";
const char* const Config::MARGIN_SEL_COLOR_KEY        = "SelectionColor";
const bool        Config::DEFAULT_MARGIN_USE_DEFAULTS = true;
const bool        Config::DEFAULT_MARGIN_USE_NUMBERS  = true;
const QRgb        Config::DEFAULT_MARGIN_BKG_COLOR    = qRgb( 245, 245, 245 );
const QRgb        Config::DEFAULT_MARGIN_FONT_COLOR   = qRgb(   0,   0,   0 );
const QRgb        Config::DEFAULT_MARGIN_SEL_COLOR    = qRgb( 255, 255, 255 );
// ******* RIGHT MARGIN LINE ***************************************
const QString     Config::RMARGIN_GROUP                = "RightMarginLine";
const char* const Config::RMARGIN_USE_DEFAULTS_KEY     = "UseDefaults";
const char* const Config::RMARGIN_USE_KEY              = "Use";
const char* const Config::RMARGIN_N_KEY                = "N";
const char* const Config::RMARGIN_COLOR_KEY            = "Color";
bool              Config::DEFAULT_RMARGIN_USE_DEFAULTS = true;
bool              Config::DEFAULT_RMARGIN_USE          = true;
qint32            Config::DEFAULT_RMARGIN_N            = 80;
QRgb              Config::DEFAULT_RMARGIN_COLOR        = qRgb(  194, 170, 139 );



//*******************************************************************
// Config                                                CONSTRUCTOR
//*******************************************************************
Config::Config() : QObject()
{
   setParent( qApp );

   bookmark_icon_   = QPixmap( ":/img/bookmark.png" );
   breakpoint_icon_ = QPixmap( ":/img/stop.png" );
   read();
}
// end of Classer

//*******************************************************************
// instance                                                   PUBLIC
//*******************************************************************
Config* Config::instance()
{
   if( 0 == d_instance ) {
      d_instance = new Config();
   }
   return d_instance;
}
// end of instance

//*******************************************************************
// read                                                      PRIVATE
//*******************************************************************
void Config::read()
{
   d_last_opened_files.clear();
   d_last_opened_projects.clear();
   d_syntax.clear();
   d_syntax.resize( EndMarker );
   
   QVariant data;
   Settings stt;

   // ******* RIGHT MARGIN LINE ***************************************
   rmargin_use_defaults_ =
      stt.read( RMARGIN_GROUP + RMARGIN_USE_DEFAULTS_KEY, data )
      ? data.toBool()
      : DEFAULT_RMARGIN_USE_DEFAULTS;
   rmargin_use_ =
      stt.read( RMARGIN_GROUP + RMARGIN_USE_KEY, data )
      ? data.toBool()
      : DEFAULT_RMARGIN_USE;
   rmargin_n_ =
      stt.read( RMARGIN_GROUP + RMARGIN_N_KEY, data )
      ? data.toInt()
      : DEFAULT_RMARGIN_N;
   rmargin_color_ =
      stt.read( RMARGIN_GROUP + RMARGIN_COLOR_KEY, data )
      ? data.toUInt()
      : DEFAULT_RMARGIN_COLOR;      

   // ******* MARGINS *************************************************
   margin_use_defaults_ =
      stt.read( MARGIN_GROUP + MARGIN_USE_DEFAULTS_KEY, data )
      ? data.toBool()
      : DEFAULT_MARGIN_USE_DEFAULTS;
   margin_use_numbers_ =
      stt.read( MARGIN_GROUP + MARGIN_USE_NUMBERS_KEY, data )
      ? data.toBool()
      : DEFAULT_MARGIN_USE_NUMBERS;
   margin_font_color_ =
      stt.read( MARGIN_GROUP + MARGIN_FONT_COLOR_KEY, data )
      ? data.toUInt()
      : DEFAULT_MARGIN_FONT_COLOR;
   margin_bkg_color_ =
      stt.read( MARGIN_GROUP + MARGIN_BKG_COLOR_KEY, data )
      ? data.toUInt()
      : DEFAULT_MARGIN_BKG_COLOR;
   margin_sel_color_ =
      stt.read( MARGIN_GROUP + MARGIN_SEL_COLOR_KEY, data )
      ? data.toUInt()
      : DEFAULT_MARGIN_SEL_COLOR;

   // ******* FILE TREE ***********************************************
   d_fltree_use_defaults =
         stt.read( FLTREE_GROUP + USE_DEFAULTS_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_DEFAULTS;
   d_fltree_use_altbkg =
         stt.read( FLTREE_GROUP + USE_ALTBKG_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_ALTBKG;
   d_fltree_base_bkg =
         ( !d_fltree_use_defaults && stt.read( FLTREE_GROUP + BASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_BASE_BKG;
   d_fltree_altbase_bkg =
         ( !d_fltree_use_defaults && stt.read( FLTREE_GROUP + ALTBASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_ALTBASE_BKG;
   d_fltree_font_as_text =
         ( !d_fltree_use_defaults && stt.read( FLTREE_GROUP + FONT_KEY, data ) )
         ? data.toString()
         : "";
   d_fltree_font_color =
         ( !d_fltree_use_defaults && stt.read( FLTREE_GROUP + FONT_COLOR_KEY, data ) )
         ? data.toUInt()
         : qRgb( 0, 0, 0 );

   // ******* CLASS TREE **********************************************
   d_cltree_use_defaults =
         stt.read( CLTREE_GROUP + USE_DEFAULTS_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_DEFAULTS;
   d_cltree_use_altbkg =
         stt.read( CLTREE_GROUP + USE_ALTBKG_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_ALTBKG;
   d_cltree_base_bkg =
         ( !d_cltree_use_defaults && stt.read( CLTREE_GROUP + BASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_BASE_BKG;
   d_cltree_altbase_bkg =
         ( !d_cltree_use_defaults && stt.read( CLTREE_GROUP + ALTBASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_ALTBASE_BKG;
   d_cltree_font_as_text =
         ( !d_cltree_use_defaults && stt.read( CLTREE_GROUP + FONT_KEY, data ) )
         ? data.toString()
         : "";
   d_cltree_font_color =
         ( !d_cltree_use_defaults && stt.read( CLTREE_GROUP + FONT_COLOR_KEY, data ) )
         ? data.toUInt()
         : qRgb( 0, 0, 0 );

   // ******* BUILD TAB ***********************************************
   d_build_tab_use_defaults =
         stt.read( BUILD_TAB_GROUP + USE_DEFAULTS_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_DEFAULTS;
   d_build_tab_base_bkg =
         ( !d_build_tab_use_defaults && stt.read( BUILD_TAB_GROUP + BASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_BASE_BKG;
   d_build_tab_font_as_text =
         ( !d_build_tab_use_defaults && stt.read( BUILD_TAB_GROUP + FONT_KEY, data ) )
         ? data.toString()
         : "";
   d_build_tab_font_color =
         ( !d_build_tab_use_defaults && stt.read( BUILD_TAB_GROUP + FONT_COLOR_KEY, data ) )
         ? data.toUInt()
         : qRgb( 0, 0, 0 );

   // ******* MESSAGE TAB *********************************************
   d_msg_tab_use_defaults =
         stt.read( MSG_TAB_GROUP + USE_DEFAULTS_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_DEFAULTS;
   d_msg_tab_use_altbkg =
         stt.read( MSG_TAB_GROUP + USE_ALTBKG_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_ALTBKG;
   d_msg_tab_base_bkg =
         ( !d_msg_tab_use_defaults && stt.read( MSG_TAB_GROUP + BASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_BASE_BKG;
   d_msg_tab_altbase_bkg =
         ( !d_msg_tab_use_defaults && stt.read( MSG_TAB_GROUP + ALTBASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_ALTBASE_BKG;
   d_msg_tab_font_as_text =
         ( !d_msg_tab_use_defaults && stt.read( MSG_TAB_GROUP + FONT_KEY, data ) )
         ? data.toString()
         : "";
   d_msg_tab_font_color =
         ( !d_msg_tab_use_defaults && stt.read( MSG_TAB_GROUP + FONT_COLOR_KEY, data ) )
         ? data.toUInt()
         : qRgb( 0, 0, 0 );

   // ******* SEARCH TAB **********************************************
   d_search_tab_use_defaults =
         stt.read( SEARCH_TAB_GROUP + USE_DEFAULTS_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_DEFAULTS;
   d_search_tab_use_altbkg =
         stt.read( SEARCH_TAB_GROUP + USE_ALTBKG_KEY, data )
         ? data.toBool()
         : DEFAULT_USE_ALTBKG;
   d_search_tab_base_bkg =
         ( !d_search_tab_use_defaults && stt.read( SEARCH_TAB_GROUP + BASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_BASE_BKG;
   d_search_tab_altbase_bkg =
         ( !d_search_tab_use_defaults && stt.read( SEARCH_TAB_GROUP + ALTBASE_BKG_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_ALTBASE_BKG;
   d_search_tab_font_as_text =
         ( !d_search_tab_use_defaults && stt.read( SEARCH_TAB_GROUP + FONT_KEY, data ) )
         ? data.toString()
         : "";
   d_search_tab_font_color =
         ( !d_search_tab_use_defaults && stt.read( SEARCH_TAB_GROUP + FONT_COLOR_KEY, data ) )
         ? data.toUInt()
         : qRgb( 0, 0, 0 );


   if( stt.read( MAIN_WINDOW_GROUP + MENU_FILES_KEY, data ) ) {
      d_last_opened_files = data.toStringList();
   }
   if( stt.read( MAIN_WINDOW_GROUP + MENU_PROJECTS_KEY, data ) ) {
      d_last_opened_projects = data.toStringList();
   } 

   // ******* EDITOR **************************************************
   d_load_project =
         stt.read( PROPERTIES_GROUP + LOAD_PROJECT_KEY, data )
         ? data.toBool()
         : DEFAULT_PROJECT_LOAD;
   d_load_files =
         stt.read( PROPERTIES_GROUP + LOAD_FILES_KEY, data )
         ? data.toBool()
         : DEFAULT_FILES_LOAD;
   qmake_path_ =
         stt.read( PROPERTIES_GROUP + QT_MAKE_PATH_KEY, data )
         ? data.toString()
         : DEFAULT_QMAKE_PATH;
   before_qmake_ =
         stt.read( PROPERTIES_GROUP + BEFORE_QT_MAKE_KEY, data )
         ? data.toStringList()
         : QStringList();
   after_qmake_ =
         stt.read( PROPERTIES_GROUP + AFTER_QT_MAKE_KEY, data )
         ? data.toStringList()
         : QStringList();
   d_parse_class_onsave =
         stt.read( PROPERTIES_GROUP + PARSE_CLASS_KEY, data )
         ? data.toBool()
         : DEFAULT_PARSE_CLASS_ONSAVE;
   d_is_default_edit_config =
         stt.read( PROPERTIES_GROUP + EDITOR_DEFAULT_KEY, data )
         ? data.toBool()
         : true;
   d_is_line_highlight =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_HIGHLIGHT_LINE_KEY, data ) )
         ? data.toBool()
         : DEFAULT_HIGHLIGHT_LINE;
   d_tab_width =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_TAB_WIDTH_KEY, data ) )
         ? data.toInt()
         : DEFAULT_TAB_WIDTH;
   d_is_auto_indent =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_AUTO_INDENT_KEY, data ) )
         ? data.toBool()
         : DEFAULT_AUTO_INDENT;
   d_is_syntax_highlight =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_SYNTAX_HIGHLIGHT_KEY, data ) )
         ? data.toBool()
         : DEFAULT_SYNTAX_HIGHLIGHT;
   d_cpp_background =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + CPP_BACKGROUND_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_CPP_BACKGROUND;
   d_hpp_background =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + HPP_BACKGROUND_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_HPP_BACKGROUND;
   d_currline_background =
         ( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + CURRLINE_BACKGROUND_KEY, data ) )
         ? data.toUInt()
         : DEFAULT_CURRLINE_BACKGROUND;
   

   // ******* TEXT ******************************************   
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_TEXT_PROP_KEY, data ) )  {
      d_syntax[ Text ] = data.toList();
   }
   if( d_syntax[ Text ].isEmpty() ) {
      d_syntax[ Text ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Text ].append( DEFAULT_TEXT_COLOR );
   }
   // ******* COMMENT ***************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_COMMENT_PROP_KEY, data ) ) {
      d_syntax[ Comment ] = data.toList();
   }
   if( d_syntax[ Comment ].isEmpty() ) {
      d_syntax[ Comment ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Comment ].append( DEFAULT_COMMENT_COLOR );
   }
   // ******* STRING ****************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_STRING_PROP_KEY, data ) ) {
      d_syntax[ String ] = data.toList();
   }
   if( d_syntax[ String ].isEmpty() ) {
      d_syntax[ String ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ String ].append( DEFAULT_STRING_COLOR );
   }
   // ******* CHAR ******************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_CHAR_PROP_KEY, data ) ) {
      d_syntax[ Char ] = data.toList();
   }
   if( d_syntax[ Char ].isEmpty() ) {
      d_syntax[ Char ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Char ].append( DEFAULT_CHAR_COLOR );
   }
   // ******* DECNUM ****************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_DECNUM_PROP_KEY, data ) ) {
      d_syntax[ DecNum ] = data.toList();
   }
   if( d_syntax[ DecNum ].isEmpty() ) {
      d_syntax[ DecNum ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ DecNum ].append( DEFAULT_DECNUM_COLOR );
   }
   // ******* HEXNUM ****************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_HEXNUM_PROP_KEY, data ) ) {
      d_syntax[ HexNum ] = data.toList();
   }
   if( d_syntax[ HexNum ].isEmpty() ) {
      d_syntax[ HexNum ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ HexNum ].append( DEFAULT_HEXNUM_COLOR );
   }
   // ******* KEYWORD ***************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_KEYWORD_PROP_KEY, data ) ) {
      d_syntax[ Keyword ] = data.toList();
   }
   if( d_syntax[ Keyword ].isEmpty() ) {
      d_syntax[ Keyword ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Keyword ].append( DEFAULT_KEYWORD_COLOR );
   }
   // ******* TYPE ******************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_TYPE_PROP_KEY, data ) ) {
      d_syntax[ Type ] = data.toList();
   }
   if( d_syntax[ Type ].isEmpty() ) {
      d_syntax[ Type ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Type ].append( DEFAULT_TYPE_COLOR );
   }
   // ******* PRAGMA ****************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_PRAGMA_PROP_KEY, data ) ) {
      d_syntax[ Pragma ] = data.toList();
   }
   if( d_syntax[ Pragma ].isEmpty() ) {
      d_syntax[ Pragma ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Pragma ].append( DEFAULT_PRAGMA_COLOR );
   }
   // ******* CLASS *****************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_CLASS_PROP_KEY, data ) ) {
      d_syntax[ Class ] = data.toList();
   }
   if( d_syntax[ Class ].isEmpty() ) {
      d_syntax[ Class ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Class ].append( DEFAULT_CLASS_COLOR );
   }
   // ******* QT ********************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_QT_PROP_KEY, data ) ) {
      d_syntax[ Qt ] = data.toList();
   }
   if( d_syntax[ Qt ].isEmpty() ) {
      d_syntax[ Qt ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Qt ].append( DEFAULT_QT_COLOR );
   }
   // ******* STL ****************************************
   if( !d_is_default_edit_config && stt.read( PROPERTIES_GROUP + EDITOR_STL_PROP_KEY, data ) ) {
      d_syntax[ Stl ] = data.toList();
   }
   if( d_syntax[ Stl ].isEmpty() ) {
      d_syntax[ Stl ].append( DEFAULT_EDITOR_TEXT_FONT );
      d_syntax[ Stl ].append( DEFAULT_STL_COLOR );
   }
   // ******* TAB / SPACES **********************************
   tab2spc_ =
         stt.read( PROPERTIES_GROUP + EDITOR_TAB2SPC_KEY, data )
         ? data.toBool()
         : true;
}
// end of read

//*******************************************************************
// save                                                      PRIVATE
//*******************************************************************
void Config::save()
{
   Settings stt;

   // ******* GENERAL PROPERTIES **************************************
   stt.save( PROPERTIES_GROUP + LOAD_PROJECT_KEY           , d_load_project           );
   stt.save( PROPERTIES_GROUP + LOAD_FILES_KEY             , d_load_files             );
   stt.save( PROPERTIES_GROUP + QT_MAKE_PATH_KEY           , qmake_path_              );
   stt.save( PROPERTIES_GROUP + BEFORE_QT_MAKE_KEY         , before_qmake_            );
   stt.save( PROPERTIES_GROUP + AFTER_QT_MAKE_KEY          , after_qmake_             );
   stt.save( PROPERTIES_GROUP + PARSE_CLASS_KEY            , d_parse_class_onsave     );
   stt.save( PROPERTIES_GROUP + EDITOR_DEFAULT_KEY         , d_is_default_edit_config );
   stt.save( PROPERTIES_GROUP + EDITOR_HIGHLIGHT_LINE_KEY  , d_is_line_highlight      );
   stt.save( PROPERTIES_GROUP + EDITOR_TAB_WIDTH_KEY       , d_tab_width              );
   stt.save( PROPERTIES_GROUP + EDITOR_AUTO_INDENT_KEY     , d_is_auto_indent         );
   stt.save( PROPERTIES_GROUP + EDITOR_SYNTAX_HIGHLIGHT_KEY, d_is_syntax_highlight    );
   stt.save( PROPERTIES_GROUP + CPP_BACKGROUND_KEY         , d_cpp_background         );
   stt.save( PROPERTIES_GROUP + HPP_BACKGROUND_KEY         , d_hpp_background         );
   stt.save( PROPERTIES_GROUP + CURRLINE_BACKGROUND_KEY    , d_currline_background    );
   stt.save( PROPERTIES_GROUP + EDITOR_TEXT_PROP_KEY       , d_syntax[ Text ]         );
   stt.save( PROPERTIES_GROUP + EDITOR_COMMENT_PROP_KEY    , d_syntax[ Comment ]      );
   stt.save( PROPERTIES_GROUP + EDITOR_STRING_PROP_KEY     , d_syntax[ String ]       );
   stt.save( PROPERTIES_GROUP + EDITOR_CHAR_PROP_KEY       , d_syntax[ Char ]         );
   stt.save( PROPERTIES_GROUP + EDITOR_DECNUM_PROP_KEY     , d_syntax[ DecNum ]       );
   stt.save( PROPERTIES_GROUP + EDITOR_HEXNUM_PROP_KEY     , d_syntax[ HexNum ]       );
   stt.save( PROPERTIES_GROUP + EDITOR_KEYWORD_PROP_KEY    , d_syntax[ Keyword ]      );
   stt.save( PROPERTIES_GROUP + EDITOR_TYPE_PROP_KEY       , d_syntax[ Type ]         );
   stt.save( PROPERTIES_GROUP + EDITOR_PRAGMA_PROP_KEY     , d_syntax[ Pragma ]       );
   stt.save( PROPERTIES_GROUP + EDITOR_CLASS_PROP_KEY      , d_syntax[ Class ]        );
   stt.save( PROPERTIES_GROUP + EDITOR_QT_PROP_KEY         , d_syntax[ Qt ]           );
   stt.save( PROPERTIES_GROUP + EDITOR_STL_PROP_KEY        , d_syntax[ Stl ]          );
   stt.save( PROPERTIES_GROUP + EDITOR_TAB2SPC_KEY         , tab2spc_                 );

   // ******* MARGINS *************************************************
   stt.save( MARGIN_GROUP + MARGIN_USE_DEFAULTS_KEY, margin_use_defaults_    );
   stt.save( MARGIN_GROUP + MARGIN_USE_NUMBERS_KEY , margin_use_numbers_     );
   stt.save( MARGIN_GROUP + MARGIN_FONT_COLOR_KEY  , margin_font_color_      );
   stt.save( MARGIN_GROUP + MARGIN_BKG_COLOR_KEY   , margin_bkg_color_       );
   stt.save( MARGIN_GROUP + MARGIN_SEL_COLOR_KEY   , margin_sel_color_       );

   // ******* RIGHT MARGIN LINE ***************************************
   stt.save( RMARGIN_GROUP + RMARGIN_USE_DEFAULTS_KEY, rmargin_use_defaults_ );
   stt.save( RMARGIN_GROUP + RMARGIN_USE_KEY         , rmargin_use_          );
   stt.save( RMARGIN_GROUP + RMARGIN_N_KEY           , rmargin_n_            );
   stt.save( RMARGIN_GROUP + RMARGIN_COLOR_KEY       , rmargin_color_        );

   // ******* FILE TREE ***********************************************
   stt.save( FLTREE_GROUP + USE_DEFAULTS_KEY     , d_fltree_use_defaults     );
   stt.save( FLTREE_GROUP + USE_ALTBKG_KEY       , d_fltree_use_altbkg       );
   stt.save( FLTREE_GROUP + BASE_BKG_KEY         , d_fltree_base_bkg         );
   stt.save( FLTREE_GROUP + ALTBASE_BKG_KEY      , d_fltree_altbase_bkg      );
   stt.save( FLTREE_GROUP + FONT_KEY             , d_fltree_font_as_text     );
   stt.save( FLTREE_GROUP + FONT_COLOR_KEY       , d_fltree_font_color       );

   // ******* CLASS TREE **********************************************
   stt.save( CLTREE_GROUP + USE_DEFAULTS_KEY     , d_cltree_use_defaults     );
   stt.save( CLTREE_GROUP + USE_ALTBKG_KEY       , d_cltree_use_altbkg       );
   stt.save( CLTREE_GROUP + BASE_BKG_KEY         , d_cltree_base_bkg         );
   stt.save( CLTREE_GROUP + ALTBASE_BKG_KEY      , d_cltree_altbase_bkg      );
   stt.save( CLTREE_GROUP + FONT_KEY             , d_cltree_font_as_text     );
   stt.save( CLTREE_GROUP + FONT_COLOR_KEY       , d_cltree_font_color       );

   // ******* BUILD TAB ***********************************************
   stt.save( BUILD_TAB_GROUP + USE_DEFAULTS_KEY  , d_build_tab_use_defaults );
   stt.save( BUILD_TAB_GROUP + BASE_BKG_KEY      , d_build_tab_base_bkg     );
   stt.save( BUILD_TAB_GROUP + FONT_KEY          , d_build_tab_font_as_text );
   stt.save( BUILD_TAB_GROUP + FONT_COLOR_KEY    , d_build_tab_font_color   );

   // ******* MESSAGE TAB *********************************************
   stt.save( MSG_TAB_GROUP + USE_DEFAULTS_KEY    , d_msg_tab_use_defaults );
   stt.save( MSG_TAB_GROUP + USE_ALTBKG_KEY      , d_msg_tab_use_altbkg   );
   stt.save( MSG_TAB_GROUP + BASE_BKG_KEY        , d_msg_tab_base_bkg     );
   stt.save( MSG_TAB_GROUP + ALTBASE_BKG_KEY     , d_msg_tab_altbase_bkg  );
   stt.save( MSG_TAB_GROUP + FONT_KEY            , d_msg_tab_font_as_text );
   stt.save( MSG_TAB_GROUP + FONT_COLOR_KEY      , d_msg_tab_font_color   );

   // ******* SEARCH TAB **********************************************
   stt.save( SEARCH_TAB_GROUP + USE_DEFAULTS_KEY , d_search_tab_use_defaults );
   stt.save( SEARCH_TAB_GROUP + USE_ALTBKG_KEY   , d_search_tab_use_altbkg   );
   stt.save( SEARCH_TAB_GROUP + BASE_BKG_KEY     , d_search_tab_base_bkg     );
   stt.save( SEARCH_TAB_GROUP + ALTBASE_BKG_KEY  , d_search_tab_altbase_bkg  );
   stt.save( SEARCH_TAB_GROUP + FONT_KEY         , d_search_tab_font_as_text );
   stt.save( SEARCH_TAB_GROUP + FONT_COLOR_KEY   , d_search_tab_font_color   );
}
// end of save

