/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Config.h
 * DATE   : 24.10.2006
 *******************************************************************/
#ifndef INCLUDED_Config_h
#define INCLUDED_Config_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QObject>
#include <QString>
#include <QVariant>
#include <QStringList>
#include <QVector>
#include <QRgb>
#include <QPixmap>


/*------- class declaration:
-------------------------------------------------------------------*/
class Config : public QObject
{
// ******* CONSTRUCTION / DESTRUCTION *******
private:
	Config();
private:
	Config( const Config& );
	Config& operator=( const Config& );
	
// ******* TYPES *******
public:
	typedef QList<QVariant> SyntaxInfo;
	enum {
		Text = 0,
		Comment,
		String,
		Char,
		DecNum,
		HexNum,
		Keyword,
		Type,
		Pragma,
		Class,
		Qt,
		Stl,
		EndMarker
	};

public:
	static const bool        DEFAULT_PROJECT_LOAD;
	static const bool        DEFAULT_FILES_LOAD;
	static const char* const DEFAULT_QMAKE_PATH;
	static const bool        DEFAULT_PARSE_CLASS_ONSAVE;
	static const char* const DEFAULT_EDITOR_TEXT_FONT;
	static const bool        DEFAULT_HIGHLIGHT_LINE;
	static const qint32      DEFAULT_TAB_WIDTH;
	static const bool        DEFAULT_AUTO_INDENT;
	static const bool        DEFAULT_SYNTAX_HIGHLIGHT;

	// Goto Line
	static const QString     GOTO_LINE_GROUP;
	static const char* const GOTO_LINE_KEY;
	// Find - Replace
	static const QString     FIND_GROUP;
	static const char* const FIND_KEY;
	static const char* const REPLACE_KEY;
	static const char* const CONFIG_KEY;
	// FileTree, ClassTree, MsgTab, SearchTab
	static const QString     FLTREE_GROUP;
	static const QString     CLTREE_GROUP;
	static const QString     BUILD_TAB_GROUP;
	static const QString     SEARCH_TAB_GROUP;
	static const QString     MSG_TAB_GROUP;
	static const char* const USE_DEFAULTS_KEY;
	static const char* const USE_ALTBKG_KEY;
	static const char* const BASE_BKG_KEY;
	static const char* const ALTBASE_BKG_KEY;
	static const char* const FONT_KEY;
	static const char* const FONT_COLOR_KEY;
	
	// Groups
	static const QString     MAIN_WINDOW_GROUP;
	static const QString     PROPERTIES_GROUP;
	
	static const char* const LOAD_PROJECT_KEY;
	static const char* const LOAD_FILES_KEY;
	static const char* const QT_MAKE_PATH_KEY;
	static const char* const BEFORE_QT_MAKE_KEY;
	static const char* const AFTER_QT_MAKE_KEY;
	static const char* const PARSE_CLASS_KEY;
	static const char* const MENU_FILES_KEY;
	static const char* const MENU_PROJECTS_KEY;
	static const char* const LAST_OPENED_WINDOWS_KEY1;
	static const char* const LAST_OPENED_WINDOWS_KEY2;
	static const char* const EDITOR_DEFAULT_KEY;
	static const char* const EDITOR_HIGHLIGHT_LINE_KEY;
	static const char* const EDITOR_TAB_WIDTH_KEY;
	static const char* const EDITOR_AUTO_INDENT_KEY;
	static const char* const EDITOR_SYNTAX_HIGHLIGHT_KEY;
	static const char* const EDITOR_TEXT_PROP_KEY;
	static const char* const EDITOR_COMMENT_PROP_KEY;
	static const char* const EDITOR_STRING_PROP_KEY;
	static const char* const EDITOR_CHAR_PROP_KEY;
	static const char* const EDITOR_DECNUM_PROP_KEY;
	static const char* const EDITOR_HEXNUM_PROP_KEY;
	static const char* const EDITOR_KEYWORD_PROP_KEY;
	static const char* const EDITOR_TYPE_PROP_KEY;
	static const char* const EDITOR_PRAGMA_PROP_KEY;
	static const char* const EDITOR_CLASS_PROP_KEY;
	static const char* const EDITOR_QT_PROP_KEY;
	static const char* const EDITOR_STL_PROP_KEY;
	static const char* const EDITOR_TAB2SPC_KEY;
	static const char* const CPP_BACKGROUND_KEY;
	static const char* const HPP_BACKGROUND_KEY;
	static const char* const CURRLINE_BACKGROUND_KEY;

	static const QRgb        DEFAULT_CPP_BACKGROUND;
	static const QRgb        DEFAULT_HPP_BACKGROUND;
	static const QRgb        DEFAULT_CURRLINE_BACKGROUND;
	static const QRgb        DEFAULT_TEXT_COLOR;
	static const QRgb        DEFAULT_COMMENT_COLOR;
	static const QRgb        DEFAULT_STRING_COLOR;
	static const QRgb        DEFAULT_CHAR_COLOR;
	static const QRgb        DEFAULT_DECNUM_COLOR;
	static const QRgb        DEFAULT_HEXNUM_COLOR;
	static const QRgb        DEFAULT_KEYWORD_COLOR;
	static const QRgb        DEFAULT_TYPE_COLOR;
	static const QRgb        DEFAULT_PRAGMA_COLOR;
	static const QRgb        DEFAULT_CLASS_COLOR;
	static const QRgb        DEFAULT_STL_COLOR;
	static const QRgb        DEFAULT_QT_COLOR;
	// FileTree, ClassTree, SearchTab
	static const bool        DEFAULT_USE_DEFAULTS;
	static const bool        DEFAULT_USE_ALTBKG;
	static const QRgb        DEFAULT_BASE_BKG;
	static const QRgb        DEFAULT_ALTBASE_BKG;
	// ******* MARGIN **************************************************
	static const QString     MARGIN_GROUP;
	static const char* const MARGIN_USE_DEFAULTS_KEY;
	static const char* const MARGIN_USE_NUMBERS_KEY;
	static const char* const MARGIN_FONT_COLOR_KEY;
	static const char* const MARGIN_BKG_COLOR_KEY;
	static const char* const MARGIN_SEL_COLOR_KEY;
	static const bool        DEFAULT_MARGIN_USE_DEFAULTS;
	static const bool        DEFAULT_MARGIN_USE_NUMBERS;
	static const QRgb        DEFAULT_MARGIN_BKG_COLOR;
	static const QRgb        DEFAULT_MARGIN_FONT_COLOR;
	static const QRgb        DEFAULT_MARGIN_SEL_COLOR;
	// ******* RIGHT MARGIN LINE ***************************************
	static const QString     RMARGIN_GROUP;
	static const char* const RMARGIN_USE_DEFAULTS_KEY;
	static const char* const RMARGIN_USE_KEY;
	static const char* const RMARGIN_N_KEY;
	static const char* const RMARGIN_COLOR_KEY;
	static bool              DEFAULT_RMARGIN_USE_DEFAULTS;
	static bool              DEFAULT_RMARGIN_USE;
	static qint32            DEFAULT_RMARGIN_N;
	static QRgb              DEFAULT_RMARGIN_COLOR;            
	

// ******* MEMBERS *******
public:
	QPixmap     bookmark_icon_;
	QPixmap     breakpoint_icon_;

private:
	static Config* d_instance;

	// ******* MARGIN **************************************************
	bool        margin_use_defaults_;
	bool        margin_use_numbers_;
	QRgb        margin_font_color_;
	QRgb        margin_bkg_color_;
	QRgb        margin_sel_color_;

	// ******* RIGHT MARGIN LINE ***************************************
	bool        rmargin_use_defaults_;
	bool        rmargin_use_;
	qint32      rmargin_n_;
	QRgb        rmargin_color_;

	// FileTree
	bool        d_fltree_use_defaults;
	bool        d_fltree_use_altbkg;
	QRgb        d_fltree_base_bkg;
	QRgb        d_fltree_altbase_bkg;
	QString     d_fltree_font_as_text;
	QRgb        d_fltree_font_color;
	// ClassTree
	bool        d_cltree_use_defaults;
	bool        d_cltree_use_altbkg;
	QRgb        d_cltree_base_bkg;
	QRgb        d_cltree_altbase_bkg;
	QString     d_cltree_font_as_text;
	QRgb        d_cltree_font_color;
	// BuildTab
	bool        d_build_tab_use_defaults;
	QRgb        d_build_tab_base_bkg;
	QString     d_build_tab_font_as_text;
	QRgb        d_build_tab_font_color;
	// MsgTab
	bool        d_msg_tab_use_defaults;
	bool        d_msg_tab_use_altbkg;
	QRgb        d_msg_tab_base_bkg;
	QRgb        d_msg_tab_altbase_bkg;
	QString     d_msg_tab_font_as_text;
	QRgb        d_msg_tab_font_color;
	// SearchTab
	bool        d_search_tab_use_defaults;
	bool        d_search_tab_use_altbkg;
	QRgb        d_search_tab_base_bkg;
	QRgb        d_search_tab_altbase_bkg;
	QString     d_search_tab_font_as_text;
	QRgb        d_search_tab_font_color;
	
	// EDITOR
	bool        d_load_project;				// czy ladowac projekt przy starcie
	bool        d_load_files;				// czy ladowac pliki przy starcie
	QString     qmake_path_;
	QStringList before_qmake_;
	QStringList after_qmake_;
	bool        d_parse_class_onsave;       // czy parsing jest po zapisie pliku
	QStringList d_last_opened_files;		// ostatnio otwarte pliki
	QStringList d_last_opened_projects;		// ostatnio otwarte projekty
	bool        d_is_default_edit_config;	// czy w edytorze stosowac ustawienia domyslne
	bool        d_is_line_highlight;		// czy aktualna linia ma byc podswietlana
	qint32      d_tab_width;				// szerokosc tab
	bool        d_is_auto_indent;			// czy ma byc auto indent
	bool        d_is_syntax_highlight;		// czy podswietlac syntaktyke
	QRgb        d_cpp_background;			// kolor tla pliku zrodlowego
	QRgb        d_hpp_background;           // kolor tla pliku naglowkowego
	QRgb        d_currline_background;      // tlo aktualnej linii w edytorze
	QVector< SyntaxInfo > d_syntax;			// ustawienia syntaktyki
	bool        tab2spc_;

// ******* METHODS *******
public:
	static Config* instance ();
	void               save ();
	void               read ();

	// FileTree
	bool                fltree_use_defaults () const;
	void                fltree_use_defaults ( bool );
	bool                  fltree_use_altbkg () const;
	void                  fltree_use_altbkg ( bool );
	QRgb                    fltree_base_bkg () const;
	void                    fltree_base_bkg ( QRgb );
	QRgb                 fltree_altbase_bkg () const;
	void                 fltree_altbase_bkg ( QRgb );
	const QString&              fltree_font () const;
	void                        fltree_font ( const QString& );
	QRgb                  fltree_font_color () const;
	void                  fltree_font_color ( QRgb );
	// ClassTree
	bool                cltree_use_defaults () const;
	void                cltree_use_defaults ( bool );
	bool                  cltree_use_altbkg () const;
	void                  cltree_use_altbkg ( bool );
	QRgb                    cltree_base_bkg () const;
	void                    cltree_base_bkg ( QRgb );
	QRgb                 cltree_altbase_bkg () const;
	void                 cltree_altbase_bkg ( QRgb );
	const QString&              cltree_font () const;
	void                        cltree_font ( const QString& );
	QRgb                  cltree_font_color () const;
	void                  cltree_font_color ( QRgb );
	// BuildTab
	bool             build_tab_use_defaults () const;
	void             build_tab_use_defaults ( bool );
	QRgb                 build_tab_base_bkg () const;
	void                 build_tab_base_bkg ( QRgb );
	const QString&           build_tab_font () const;
	void                     build_tab_font ( const QString& );
	QRgb               build_tab_font_color () const;
	void               build_tab_font_color ( QRgb );
	// MsgTab
	bool               msg_tab_use_defaults () const;
	void               msg_tab_use_defaults ( bool );
	bool                 msg_tab_use_altbkg () const;
	void                 msg_tab_use_altbkg ( bool );
	QRgb                   msg_tab_base_bkg () const;
	void                   msg_tab_base_bkg ( QRgb );
	QRgb                msg_tab_altbase_bkg () const;
	void                msg_tab_altbase_bkg ( QRgb );
	const QString&             msg_tab_font () const;
	void                       msg_tab_font ( const QString& );
	QRgb                 msg_tab_font_color () const;
	void                 msg_tab_font_color ( QRgb );
	// SearchTab
	bool            search_tab_use_defaults () const;
	void            search_tab_use_defaults ( bool );
	bool              search_tab_use_altbkg () const;
	void              search_tab_use_altbkg ( bool );
	QRgb                search_tab_base_bkg () const;
	void                search_tab_base_bkg ( QRgb );
	QRgb             search_tab_altbase_bkg () const;
	void             search_tab_altbase_bkg ( QRgb );
	const QString&          search_tab_font () const;
	void                    search_tab_font ( const QString& );
	QRgb              search_tab_font_color () const;
	void              search_tab_font_color ( QRgb );

	// ******* MARGINS *************************************************
	bool margin_get_use_defaults () const;
	void margin_set_use_defaults ( bool );
	bool margin_get_use_numbers  ( bool = false ) const;
	void margin_set_use_numbers  ( bool );
	QRgb margin_get_font_color   ( bool = false ) const;
	void margin_set_font_color   ( QRgb );
	QRgb margin_get_bkg_color    ( bool = false ) const;
	void margin_set_bkg_color    ( QRgb );
	QRgb margin_get_sel_color    ( bool = false ) const;
	void margin_set_sel_color    ( QRgb );

	// ******* RIGHT MARGIN LINE ***************************************
	bool   rmargin_get_use_defaults () const;
	void   rmargin_set_use_defaults ( bool );
	bool   rmargin_get_use          ( bool = false ) const;
	void   rmargin_set_use          ( bool );	
	qint32 rmargin_get_n            ( bool = false ) const;
	void   rmargin_set_n            ( qint32 );
	QRgb   rmargin_get_color        ( bool = false ) const;
	void   rmargin_set_color        ( QRgb );
	
	// ******* EDITOR **************************************************
	QRgb                     cpp_background () const;
	void                     cpp_background ( QRgb );
	QRgb                     hpp_background () const;
	void                     hpp_background ( QRgb );
	QRgb                currline_background () const;
	void                currline_background ( QRgb );
	bool                       load_project () const;
	void                       load_project ( bool );
	bool                         load_files () const;
	void                         load_files ( bool );
	const QString&               qmake_path () const;
	void                         qmake_path ( const QString& );
	const QStringList&         qmake_before () const;
	void                       qmake_before ( const QStringList& );
	const QStringList&         qmake_after  () const;
	void                       qmake_after  ( const QStringList& );
	bool                 parse_class_onsave () const;
	bool                 parse_class_online () const;
	void                 parse_class_onsave ( bool );
	const QStringList&    last_opened_files () const;
	void                  last_opened_files ( const QStringList& );
	const QStringList& last_opened_projects () const;
	void               last_opened_projects ( const QStringList& );
	bool                default_edit_config () const;
	void                default_edit_config ( bool );
	bool                     line_highlight () const;
	void                     line_highlight ( bool );
	qint32                        tab_width () const;
	void                          tab_width ( qint32 );
	bool                        auto_indent () const;
	void                        auto_indent ( bool );
	bool                   syntax_highlight () const;
	void                   syntax_highlight ( bool );
	const QVector< SyntaxInfo >&     syntax () const;
	void                             syntax ( const QVector< SyntaxInfo >& );
	bool                            tab2spc () const;
	void                            tab2spc ( bool );
};


//###################################################################
//#                                                                 #
//#             I N L I N E   F U N C T I O N S                     #
//#                                                                 #
//###################################################################

// ******* RIGHT MARGIN LINE ***************************************
inline bool Config::rmargin_get_use_defaults () const {
	return rmargin_use_defaults_;
}
inline void Config::rmargin_set_use_defaults( const bool in_flag ) {
	rmargin_use_defaults_ = in_flag;
}
inline bool Config::rmargin_get_use( const bool in_strict ) const {
	bool retval = ( rmargin_use_defaults_ ) ? DEFAULT_RMARGIN_USE : rmargin_use_;
	if( in_strict ) {
		retval = rmargin_use_;
	}
	return retval;
}
inline void Config::rmargin_set_use( const bool in_flag ) {
	rmargin_use_ = in_flag;
}
inline qint32 Config::rmargin_get_n( const bool in_strict ) const {
	qint32 retval = ( rmargin_use_defaults_ ) ? DEFAULT_RMARGIN_N : rmargin_n_;
	if( in_strict ) {
		retval = rmargin_n_;
	}
	return retval;
}
inline void Config::rmargin_set_n( const qint32 in_value ) {
	rmargin_n_ = in_value;
}
inline QRgb Config::rmargin_get_color( const bool in_strict ) const {
	QRgb retval = ( rmargin_use_defaults_ ) ? DEFAULT_RMARGIN_COLOR : rmargin_color_;
	if( in_strict ) {
		retval = rmargin_color_;
	}
	return retval;
}
inline void Config::rmargin_set_color( const QRgb in_value ) {
	rmargin_color_ = in_value;
}

// ******* MARGINS *************************************************
inline bool Config::margin_get_use_defaults() const {
	return margin_use_defaults_;
}
inline void Config::margin_set_use_defaults( const bool in_flag ) {
	margin_use_defaults_ = in_flag;
}
inline bool Config::margin_get_use_numbers( const bool in_strict ) const {
	bool retval = ( margin_use_defaults_ ) ? DEFAULT_MARGIN_USE_NUMBERS : margin_use_numbers_;
	if( in_strict ) {
		retval = margin_use_numbers_;
	}
	return retval;
}
inline void Config::margin_set_use_numbers( const bool in_flag ) {
	margin_use_numbers_ = in_flag;
}
inline QRgb Config::margin_get_font_color( const bool in_strict ) const {
	QRgb retval = ( margin_use_defaults_ ) ? DEFAULT_MARGIN_FONT_COLOR : margin_font_color_;
	if( in_strict ) {
		retval = margin_font_color_;
	}
	return retval;
}
inline void Config::margin_set_font_color( const QRgb in_value ) {
	margin_font_color_ = in_value;
}
inline QRgb Config::margin_get_bkg_color( const bool in_strict ) const {
	QRgb retval = ( margin_use_defaults_ ) ? DEFAULT_MARGIN_BKG_COLOR : margin_bkg_color_;
	if( in_strict ) {
		retval = margin_bkg_color_;
	}
	return retval;
}
inline void Config::margin_set_bkg_color( const QRgb in_value ) {
	margin_bkg_color_ = in_value;
}
inline QRgb Config::margin_get_sel_color( const bool in_strict ) const {
	QRgb retval = ( margin_use_defaults_ ) ? DEFAULT_MARGIN_SEL_COLOR : margin_sel_color_;
	if( in_strict ) {
		retval = margin_sel_color_;
	}
	return retval;
}
inline void Config::margin_set_sel_color( const QRgb in_value ) {
	margin_sel_color_ = in_value;
}

// FileTree
inline bool Config::fltree_use_defaults() const {
	return d_fltree_use_defaults;
}
inline void Config::fltree_use_defaults( const bool in_use ) {
	d_fltree_use_defaults = in_use;
}
inline bool Config::fltree_use_altbkg () const {
	return d_fltree_use_altbkg;
}
inline void Config::fltree_use_altbkg( const bool in_use ) {
	d_fltree_use_altbkg = in_use;
}
inline QRgb Config::fltree_base_bkg () const {
	return d_fltree_base_bkg;
}
inline void Config::fltree_base_bkg ( const QRgb in_color ) {
	d_fltree_base_bkg = in_color;
}
inline QRgb Config::fltree_altbase_bkg () const {
	return d_fltree_altbase_bkg;
}
inline void Config::fltree_altbase_bkg ( const QRgb in_color ) {
	d_fltree_altbase_bkg = in_color;
}
inline const QString& Config::fltree_font () const {
	return d_fltree_font_as_text;
}
inline void Config::fltree_font ( const QString& in_font_as_text ) {
	d_fltree_font_as_text = in_font_as_text;
}
inline QRgb Config::fltree_font_color () const {
	return d_fltree_font_color;
}
inline void Config::fltree_font_color ( const QRgb in_rgb ) {
	d_fltree_font_color = in_rgb;
}

// ClassTree
inline bool Config::cltree_use_defaults() const {
	return d_cltree_use_defaults;
}
inline void Config::cltree_use_defaults( const bool in_use ) {
	d_cltree_use_defaults = in_use;
}
inline bool Config::cltree_use_altbkg () const {
	return d_cltree_use_altbkg;
}
inline void Config::cltree_use_altbkg( const bool in_use ) {
	d_cltree_use_altbkg = in_use;
}
inline QRgb Config::cltree_base_bkg () const {
	return d_cltree_base_bkg;
}
inline void Config::cltree_base_bkg ( const QRgb in_color ) {
	d_cltree_base_bkg = in_color;
}
inline QRgb Config::cltree_altbase_bkg () const {
	return d_cltree_altbase_bkg;
}
inline void Config::cltree_altbase_bkg ( const QRgb in_color ) {
	d_cltree_altbase_bkg = in_color;
}
inline const QString& Config::cltree_font () const {
	return d_cltree_font_as_text;
}
inline void Config::cltree_font ( const QString& in_font_as_text ) {
	d_cltree_font_as_text = in_font_as_text;
}
inline QRgb Config::cltree_font_color () const {
	return d_cltree_font_color;
}
inline void Config::cltree_font_color ( const QRgb in_rgb ) {
	d_cltree_font_color = in_rgb;
}
// BuildTab
inline bool Config::build_tab_use_defaults() const {
	return d_build_tab_use_defaults;
}
inline void Config::build_tab_use_defaults( const bool in_use ) {
	d_build_tab_use_defaults = in_use;
}
inline QRgb Config::build_tab_base_bkg () const {
	return d_build_tab_base_bkg;
}
inline void Config::build_tab_base_bkg ( const QRgb in_color ) {
	d_build_tab_base_bkg = in_color;
}
inline const QString& Config::build_tab_font () const {
	return d_build_tab_font_as_text;
}
inline void Config::build_tab_font ( const QString& in_font_as_text ) {
	d_build_tab_font_as_text = in_font_as_text;
}
inline QRgb Config::build_tab_font_color () const {
	return d_build_tab_font_color;
}
inline void Config::build_tab_font_color ( const QRgb in_rgb ) {
	d_build_tab_font_color = in_rgb;
}
// MsgTab
inline bool Config::msg_tab_use_defaults() const {
	return d_msg_tab_use_defaults;
}
inline void Config::msg_tab_use_defaults( const bool in_use ) {
	d_msg_tab_use_defaults = in_use;
}
inline bool Config::msg_tab_use_altbkg () const {
	return d_msg_tab_use_altbkg;
}
inline void Config::msg_tab_use_altbkg( const bool in_use ) {
	d_msg_tab_use_altbkg = in_use;
}
inline QRgb Config::msg_tab_base_bkg () const {
	return d_msg_tab_base_bkg;
}
inline void Config::msg_tab_base_bkg ( const QRgb in_color ) {
	d_msg_tab_base_bkg = in_color;
}
inline QRgb Config::msg_tab_altbase_bkg () const {
	return d_msg_tab_altbase_bkg;
}
inline void Config::msg_tab_altbase_bkg ( const QRgb in_color ) {
	d_msg_tab_altbase_bkg = in_color;
}
inline const QString& Config::msg_tab_font () const {
	return d_msg_tab_font_as_text;
}
inline void Config::msg_tab_font ( const QString& in_font_as_text ) {
	d_msg_tab_font_as_text = in_font_as_text;
}
inline QRgb Config::msg_tab_font_color () const {
	return d_msg_tab_font_color;
}
inline void Config::msg_tab_font_color ( const QRgb in_rgb ) {
	d_msg_tab_font_color = in_rgb;
}

// SearchTab
inline bool Config::search_tab_use_defaults() const {
	return d_search_tab_use_defaults;
}
inline void Config::search_tab_use_defaults( const bool in_use ) {
	d_search_tab_use_defaults = in_use;
}
inline bool Config::search_tab_use_altbkg () const {
	return d_search_tab_use_altbkg;
}
inline void Config::search_tab_use_altbkg( const bool in_use ) {
	d_search_tab_use_altbkg = in_use;
}
inline QRgb Config::search_tab_base_bkg () const {
	return d_search_tab_base_bkg;
}
inline void Config::search_tab_base_bkg ( const QRgb in_color ) {
	d_search_tab_base_bkg = in_color;
}
inline QRgb Config::search_tab_altbase_bkg () const {
	return d_search_tab_altbase_bkg;
}
inline void Config::search_tab_altbase_bkg ( const QRgb in_color ) {
	d_search_tab_altbase_bkg = in_color;
}
inline const QString& Config::search_tab_font () const {
	return d_search_tab_font_as_text;
}
inline void Config::search_tab_font ( const QString& in_font_as_text ) {
	d_search_tab_font_as_text = in_font_as_text;
}
inline QRgb Config::search_tab_font_color () const {
	return d_search_tab_font_color;
}
inline void Config::search_tab_font_color ( const QRgb in_rgb ) {
	d_search_tab_font_color = in_rgb;
}

	
// EDITOR
inline QRgb Config::cpp_background() const {
	return d_cpp_background;
}
inline void Config::cpp_background( const QRgb in_background ) {
	d_cpp_background = in_background;
}
inline QRgb Config::hpp_background() const {
	return d_hpp_background;
}
inline void Config::hpp_background( const QRgb in_background ) {
	d_hpp_background = in_background;
}

inline QRgb Config::currline_background() const {
	return d_currline_background;
}
inline void Config::currline_background( const QRgb in_background ) {
	d_currline_background = in_background;
}
inline bool Config::load_project() const {
	return d_load_project;
}
inline void Config::load_project( const bool in_load_project ) {
	d_load_project = in_load_project;
}
inline bool Config::load_files() const {
	return d_load_files;
}
inline void Config::load_files( const bool in_load_files ) {
	d_load_files = in_load_files;
}
inline const QString& Config::qmake_path() const {
	return qmake_path_;
}
inline void Config::qmake_path( const QString& in_qmake_path ) {
	qmake_path_ = in_qmake_path;
}
inline const QStringList& Config::qmake_before() const {
	return before_qmake_;
}
inline void Config::qmake_before( const QStringList& in_strings ) {
	before_qmake_ = in_strings;
}
inline const QStringList& Config::qmake_after() const {
	return after_qmake_;
}
inline void Config::qmake_after( const QStringList& in_strings ) {
	after_qmake_ = in_strings;
}
inline bool Config::parse_class_onsave() const {
	return d_parse_class_onsave;
}
inline bool Config::parse_class_online() const {
	return !d_parse_class_onsave;
}
inline void Config::parse_class_onsave( const bool in_on_save ) {
	d_parse_class_onsave = in_on_save;
}
inline const QStringList& Config::last_opened_files() const {
	return d_last_opened_files;
}
inline void Config::last_opened_files( const QStringList& in_files ) {
	d_last_opened_files = in_files;
}
inline const QStringList& Config::last_opened_projects() const {
	return d_last_opened_projects;
}
inline void Config::last_opened_projects( const QStringList& in_projects ) {
	d_last_opened_projects = in_projects;
}
inline bool Config::default_edit_config() const {
	return d_is_default_edit_config;
}
inline void Config::default_edit_config( const bool in_is_default_edit_config ) {
	d_is_default_edit_config = in_is_default_edit_config;
}
inline bool Config::line_highlight() const {
	return d_is_line_highlight;
}
inline void Config::line_highlight( const bool in_is_line_highlight ) {
	d_is_line_highlight = in_is_line_highlight;
}
inline qint32 Config::tab_width() const {
	return d_tab_width;
}
inline void Config::tab_width( const qint32 in_tab_width) {
	d_tab_width = in_tab_width;
}
inline bool Config::auto_indent() const {
	return d_is_auto_indent;
}
inline void Config::auto_indent( const bool in_is_auto_indent ) {
	d_is_auto_indent = in_is_auto_indent;
}
inline bool Config::syntax_highlight() const {
	return d_is_syntax_highlight;
}
inline void Config::syntax_highlight( const bool in_is_syntax_highlight ) {
	d_is_syntax_highlight = in_is_syntax_highlight;
}
inline const QVector< Config::SyntaxInfo >& Config::syntax() const {
	return d_syntax;
}
inline void Config::syntax( const QVector< /*Config::*/SyntaxInfo >& in_syntax ) {
	d_syntax = in_syntax;
}
inline bool Config::tab2spc() const {
	return tab2spc_;
}
inline void Config::tab2spc( const bool in_flag ) {
	tab2spc_ = in_flag;
}

#endif // INCLUDED_Config_h
