/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : CppSyntax.cpp
 * DATE   : 13.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "CppSyntax.h"  // a to jest komentarz
#include "Config.h"
#include <QBtParser.h>
#include <QTextDocument>
#include <iostream>
#include <string>
#include <vector>
using namespace std;

// STL algorytmy - szablony funkcji
const char* const CppSyntax::STLS [] = {
   "std"                     , "string"          ,
   "list"                    , "vector"          ,
   "set"                     , "multiset"        ,
   "queue"                   , "priority_queue"  ,
   "stack"                   , "map"             ,
   "multimap"                , "deque"           ,
   "complex"                 , "bitset"          ,
   "cin"                     , "cout"            ,
   "cerr"                    , "clog"            ,
   "endl"                    , "ends"            };

const char* const CppSyntax::TYPES []          = {
   "bool"                    , "char"            ,
   "const"                   , "double"          ,
   "enum"                    , "explicit"        ,
   "extern"                  , "float"           ,
   "friend"                  , "inline"          ,
   "int"                     , "long"            ,
   "mutable"                 , "register"        ,
   "short"                   , "signed"          ,
   "static"                  , "struct"          ,
   "template"                , "union"           ,
   "unsigned"                , "virtual"         ,
   "void"                    , "volatile"        ,
   "wchar_t"                 , "qint8"           ,
   "qint16"                  , "qint32"          ,
   "qint64"                  , "quint8"          ,
   "quint16"                 , "quint32"         ,
   "quint64"                 , "qreal"           ,
   "uchar"                   , "uint"            ,
   "ulong"                   , "ushort"          };

const char* const CppSyntax::KEYWORDS []       = {
   "asm"                     , "auto"            ,
   "break"                   , "case"            ,
   "catch"                   , "class"           ,
   "const_cast"              , "continue"        ,
   "default"                 , "delete"          ,
   "do"                      , "dynamic_cast"    ,
   "else"                    , "export"          , 
   "false"                   , "for"             ,
   "goto"                    , "if"              ,
   "namespace"               , "new"             ,
   "operator"                , "private"         ,
   "protected"               , "public"          ,
   "reinterpret_cast"        , "return"          ,
   "sizeof"                  , "static_cast"     ,
   "switch"                  , "this"            ,
   "throw"                   , "true"            ,
   "try"                     , "typedef"         ,
   "typeid"                  , "typename"        ,
   "using"                   , "while"           ,
   "foreach"                                     };

const char* const CppSyntax::QT []             = {
   "SIGNAL"                  , "SLOT"            ,
   "QT_TR_NOOP"              , "tr"              ,
   "qDebug"                  , "qWarning"        ,
   "qCritical"               , "qFatal"          ,
   "Q_ASSERT"                , "Q_ASSERT_X"      ,
   "Q_CHECK_PTR"             , "qSort"           ,
   "qFind"                   , "qCopy"           ,
   "qCopyBackward"           , "qBinaryFind"     ,
   "qLowerBound"             , "qUpperBound"     ,
   "qCount"                  , "qDeleteAll"      ,
   "qEqual"                  , "qFill"           ,
   "qSwap"                   , "qMax"            ,
   "qMin"                    , "qBound"          ,
   "qAbs"                    , "Qt"              };

QSet< QString >   CppSyntax::stl_set_     = QSet< QString >();
QSet< QString >   CppSyntax::type_set_    = QSet< QString >();
QSet< QString >   CppSyntax::keyword_set_ = QSet< QString >();
QSet< QString >   CppSyntax::qt_set_      = QSet< QString >();
bool              CppSyntax::initialised_ = false;

QTextCharFormat   CppSyntax::format_[ Config::EndMarker ] = { QTextCharFormat() };

//*******************************************************************
// CppSyntax                                             CONSTRUCTOR
//*******************************************************************
CppSyntax::CppSyntax( QTextDocument* const in_parent )
: QSyntaxHighlighter  ( in_parent )
{
   if( !initialised_ ) init();
}
// end of CppSyntax

//*******************************************************************
// init                                                      PRIVATE
//*******************************************************************
void CppSyntax::init()
{
   {  // STL ----------------------------------------------
      stl_set_.clear();
      const qint32 n = sizeof( STLS ) / sizeof( STLS[ 0 ] );
      for( qint32 i = 0; i < n; ++i ) {
         stl_set_ << STLS[ i ];
      }
   }
   {  // TYPES --------------------------------------------
      type_set_.clear();
      const qint32 n = sizeof( TYPES ) / sizeof( TYPES[ 0 ] );
      for( qint32 i = 0; i < n; ++i ) {
         type_set_ << TYPES[ i ];
      }
   }
   {  // KEYWORDS -----------------------------------------
      keyword_set_.clear();
      const qint32 n = sizeof( KEYWORDS ) / sizeof( KEYWORDS[ 0 ] );
      for( qint32 i = 0; i < n; ++i ) {
         keyword_set_ << KEYWORDS[ i ];
      }
   }
   {  // QT -----------------------------------------------
      qt_set_.clear();
      const qint32 n = sizeof( QT ) / sizeof( QT[ 0 ] );
      for( qint32 i = 0; i < n; ++i ) {
         qt_set_ << QT[ i ];
      }
   }
   set_properties();
   initialised_ = true;
}
// end of init

//*******************************************************************
// set_properties                                            PRIVATE
//*******************************************************************
void CppSyntax::set_properties()
{
   QFont sfont;
   const QVector< Config::SyntaxInfo >& syntax = Config::instance()->syntax();

   for( qint32 i = 0; i < Config::EndMarker; ++i ) {
      sfont.fromString( syntax[ i ][ 0 ].toString() );
      format_[ i ].setFont( sfont );
      format_[ i ].setForeground( QColor( syntax[ i ][ 1 ].toUInt() ) );
   }
}
// end of set_properties

//*******************************************************************
// rehighlight                                               PRIVATE
//*******************************************************************
void CppSyntax::rehighlight()
{
   set_properties();
   QSyntaxHighlighter::rehighlight();
}
// end of rehighlight

//*******************************************************************
// highlightBlock
//*******************************************************************
void CppSyntax::highlightBlock( const QString& in_text )
{
   qint32 status = previousBlockState();
   
   if( !in_text.isEmpty() ) {
      const qint32 n      = in_text.length();
      const qint32 ost    = n - 1;
      qint32       idx    = 0;
      QChar        znak;
      QChar        nznak;

      // NAJPIERW OBSLUGUJEMY NIE ZAKONCZONY KOMENTARZ (Z POPRZEDNIEJ LINII)
      if( 1 == status ) {
         const qint32 pos = QBtParser::search_muster( in_text, QBtParser::C_COMMENT_END );
         if( -1 == pos ) {
            save_info( Config::Comment, idx, n );
         }
         else {
            save_info( Config::Comment, idx, pos + 2 );
            status = 0;
         }
      }

      // ANALIZA PRZYSLANEGO BLOKU TEKSTU
      while( idx < n ) {
         znak = in_text[ idx ];

         // PRAGMA ===========================================
         if( '#' == znak ) {
            const qint32 pos = QBtParser::search_muster( in_text, QBtParser::CPP_COMMENT, idx );
            if( -1 == pos ) {
               save_info( Config::Pragma, idx, n - idx );
            }
            else {
               save_info( Config::Pragma , idx, pos - idx );
               save_info( Config::Comment, idx, n   - idx );
            }
            continue;
         }

         // KOMENTARZ ========================================
         if( ( QBtParser::SLASH == znak ) && ( idx < ost ) ) {
            nznak = in_text[ idx + 1 ];
            // c-komentarz
            if( QBtParser::STERN == nznak ) {
               save_info( Config::Comment, idx, 2 );
               const qint32 pos = QBtParser::search_muster( in_text, QBtParser::C_COMMENT_END, idx );
               if( -1 == pos ) {
                  save_info( Config::Comment, idx, n - idx + 1 );
                  status = 1;
               }
               else {
                  save_info( Config::Comment, idx, pos - idx + 2 );
                  status = 0;
               }
               continue;
            }
            // cpp-komentarz
            if( QBtParser::SLASH == nznak ) {
               save_info( Config::Comment, idx, n - idx );
               continue;
            }
         }

         // POJEDYNCZY ZNAK ==================================
         if( ( QBtParser::SINGLE_QUOTE == znak ) &&  QBtParser::is_acceptable( in_text, idx ) ) {
            const qint32 pos = QBtParser::search_char( in_text, QBtParser::SINGLE_QUOTE, idx + 1 );
            if( -1 == pos ) save_info( Config::Char, idx, n - idx );
            else            save_info( Config::Char, idx, pos - idx + 1 );
            continue;
         }

         // LANCUCH TEKSTOWY =================================
         if( ( QBtParser::DOUBLE_QUOTE == znak ) && QBtParser::is_acceptable( in_text, idx ) ) {
            const qint32 pos = QBtParser::search_char( in_text, QBtParser::DOUBLE_QUOTE, idx + 1 );
            if( -1 == pos ) save_info( Config::String, idx, n - idx );
            else            save_info( Config::String, idx, pos - idx + 1 );
            continue;
         }
      
         // NORMALNE ZNAKI ===================================
         if( QBtParser::is_word_begin( in_text, idx )) {
            const qint32 pos = QBtParser::search_word_end( in_text, idx + 1 );
            if( pos != -1 ) {
               qint32        type   = Config::Text;
               const qint32  length = pos - idx + 1;
               const QString word   = in_text.mid( idx, length );

                    if( QBtParser::is_dec_number( word ) ) type = Config::DecNum;
               else if( QBtParser::is_hex_number( word ) ) type = Config::HexNum;
               else if( keyword_set_.contains( word ) ) type = Config::Keyword;
               else if( type_set_.contains   ( word ) ) type = Config::Type;
               else if( stl_set_.contains    ( word ) ) type = Config::Stl;
               else if( qt_set_.contains     ( word ) ) type = Config::Qt;
               else if( is_qt                ( word ) ) type = Config::Qt;
               else if( is_class_name( in_text, pos ) ) type = Config::Class;

               save_info( type, idx, length );
            }
            // slowo bez konca
            else {
               save_info( Config::Text, idx, n );
            }
         }
         // to nie jest poczatek slowa, czyli taki jakis zwykly znak
         else {
            save_info( Config::Text, idx );
         }
      }  // end of while
   }
   setCurrentBlockState( status );  
}
// end of highlightBlock

//*******************************************************************
// save_info                                                 PRIVATE
//-------------------------------------------------------------------
// UWAGA. Nalezy zwrocic uwage na licznik inout_idx.
// PO ZAPAMIETANIU DANYCH WSKAZUJE ON NA NASTEPNY ZNAK.
//*******************************************************************
inline void CppSyntax::save_info( const qint32 in_type, qint32& inout_idx )
{
   setFormat( inout_idx, 1, format_[ in_type ] );
   ++inout_idx;
}
inline void CppSyntax::save_info( const qint32 in_type, qint32& inout_idx, const qint32 in_len )
{
   setFormat( inout_idx, in_len, format_[ in_type ] );
   inout_idx += in_len;
}
// end of save_info

//*******************************************************************
// is_class_name                                             PRIVATE
//*******************************************************************
bool CppSyntax::is_class_name( const QString& in_text, const int in_idx ) const
{
   bool retval = false;
   
   if( in_idx < ( in_text.length() - 2 ) ) {
      retval = ( ( QBtParser::COLON == in_text[ in_idx + 1 ] ) && ( QBtParser::COLON == in_text[ in_idx + 2 ] ) );
   }
   return retval;
}
// end of is_class_name

//*******************************************************************
// is_qt                                                     PRIVATE
//*******************************************************************
bool CppSyntax::is_qt( const QString& in_text ) const
{
   bool retval = false;

   if( in_text.length() > 1 ) {
      retval = ( ( 'Q' == in_text[ 0 ] ) && in_text[ 1 ].isLetterOrNumber() && in_text[ 1 ].isUpper() );
   }
   return retval;
}
// end of is_qt
