/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : DbgDataTable.h
 * DATE   : 16.02.2007
 *******************************************************************/
#ifndef INCLUDED_DbgDataTable_h
#define INCLUDED_DbgDataTable_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QTableWidget>
#include <QHash>
#include <QRgb>

/*------- class declaration:
-------------------------------------------------------------------*/
class DbgDataTable : public QTableWidget
{
	Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
	DbgDataTable( QWidget*, qint32 = 2, bool = true, qint32 = 6 );
private:
	DbgDataTable( const DbgDataTable& );
	DbgDataTable& operator=( const DbgDataTable& );

// ******* CONSTANTS *******
private:
	static const QRgb   GRAY;
	static const QRgb   NORMAL_TEXT_COLOR;
	static const QRgb   CHANGED_TEXT_COLOR;
	static const QRgb   ERROR_TEXT_COLOR;

// ******* MEMBERS *******
private:
	const qint32              d_X_width;
	qint32                    d_n_char_width;
	const qint32              d_row_height;
protected:
	qint32                    d_idx;
	QHash< QString, QString > d_prv_data;
	bool                      d_disabled_slot;
	bool                      d_set_min_width;

// ******* METHODS *******
public:
	void             showEvent ( QShowEvent* );
	void           resizeEvent ( QResizeEvent* );
	void              finished ();
	qint32             add_row ( const qint32 = -1 );
	void           remove_rows ( qint32 );
	void                colors ( qint32, bool = false );
	void              set_item ( qint32, qint32, const QString& );
	bool              get_item ( qint32, qint32, QString& );
	void          set_tool_tip ( qint32, qint32, const QString& );
	bool          get_tool_tip ( qint32, qint32, QString& );
	void        resize_columns ();
	qint32 total_columns_width ();
	void           resize_rows ();
	void          get_prv_data ();
};

#endif // INCLUDED_DbgDataTable_h
