/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : DbgMemory.h
 * DATE   : 21.02.2007
 *******************************************************************/
#ifndef INCLUDED_DbgMemory_h
#define INCLUDED_DbgMemory_h

/*------- include files:
-------------------------------------------------------------------*/
#include "DbgToolDialog.h"
#include "DbgDataTable.h"
#include <QRegExp>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QLineEdit;
class Event;

/*------- class declaration:
-------------------------------------------------------------------*/
class DbgMemory : public DbgToolDialog
{
	Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
	DbgMemory( QWidget* = 0 );
	~DbgMemory();
private:
	DbgMemory( const DbgMemory& );
	DbgMemory& operator=( const DbgMemory& );

// ******* LOCAL CONSTANTS *******
private:
	static const char* const CAPTION;
	static const char* const ADDRESS;
	static const char* const HOW_MANY;
	static const char* const HEX;
	static const char* const DEC;
	static const char* const CHAR;
	static const QString     REGEXP_MUSTER;
	static const QString     POST_PROMPT;
	static const QString     PRE_PROMPT;
	static const QString     ERROR;

// ******* MEMBERS *******
private:
	QLineEdit*    const d_addr;
	QLineEdit*    const d_how_many;
	DbgDataTable* const d_table;
	QRegExp             d_regexp;
	quint32             d_addr_value;
	quint32             d_how_many_value;
	bool                d_is_hex;
	bool                d_is_dec;
	bool                d_is_char;

// ******* METHODS *******
private:
	void         showEvent ( QShowEvent* );
	void        closeEvent ( QCloseEvent* );
	void       customEvent ( QEvent* );
	void        parse_data ( Event* );
	void      display_data ( const QStringList& );
	void            finish ();
private slots:
    void     addr_ret_slot ();
    void how_many_ret_slot ();
    void          hex_slot ();
    void          dec_slot ();
    void         char_slot ();
};

#endif // INCLUDED_DbgMemory_h
