/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Debugger.h
 * DATE   : 04.02.2007
 *******************************************************************/
#ifndef INCLUDED_Debugger_h
#define INCLUDED_Debugger_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QProcess>
#include <QVector>
#include <QMutex>

/*------- forward declarations:
-------------------------------------------------------------------*/
class Event;

/*------- class declaration:
-------------------------------------------------------------------*/
class Debugger : public QProcess
{
	Q_OBJECT
public:
	Debugger();
	~Debugger();

// ******* TYPES *******
private:
	typedef enum {
		None = 0,
		Registers,
		CallStack,
		DisAsm,
		Memory,
	} Cmd;

// ******* CONSTANTS *******
private:
	static const QChar   NEW_LINE;
	static const QChar   SPACE;
	static const QChar   CTRL_Z;
	static const QChar   COLON;
	static const QChar   COMMAS;
	static const QString PTR;
	static const QString GDB_CALL;
	static const QString GDB_RUN;
	static const QString GDB_QUIT;
	static const QString STEP;
	static const QString NEXT;
	static const QString FINISH;
	static const QString SIG_INT;
	static const QString PRINT;
	static const QString CONTINUE;
	static const QString REGISTERS;
	static const QString BACKTRACE;
	static const QString DISASSEMBLE;
	static const QString EXAMINE;
	static const QString SET_BREAKPOINT;
	static const QString CLR_BREAKPOINT;
	static const QString PROMPT_MARKER;
	static const QString SYMBOL_MARKER;
	static const QString INFO_SCOPE;
	static const QString SOURCE_MARKER;
	static const QString END_OF_PROGRAM;
	static const QString ERROR_MARKER;
	static const QString FUNCTION_MARKER;
	static const QString VALUE_MARKER;
	static const QString SCOPE_MARKER;
	static const QString THREAD_MARKER;
	static const QString PROG_START_MARKER;
	static const QString HEX_MARKER;
	static const QString SIZE_MARKER;
	static const QString DATA_MARKER;
	static const QString STD_STRING_MARKER;
	static const QString VALUE_START_MARKER;
	static const QString VALUE_END_MARKER;
	static const QString PROG_EXIT_MARKER;
	static const QString TYPE_MARKER;
	static const QString WHAT_IS;
	static const QString PRINT_STRING;
	static const QString PRINT_PTR;
	static const QString PRINT_ARROW;
	
// ******* MEMBERS *******
private:
	Cmd         d_cmd;
	QString     d_program;
	qint32      d_pid;
	bool        d_started;
	QStringList d_output;
	QVector< QStringList > d_buffer;
	QString     d_current_file;
	QString     d_current_function;
	qint32      d_current_line;
	QString     d_varname;
	QString     d_vartype;
	bool        d_registers;
	bool        d_callstack;
	bool        d_disasm;
	bool        d_memory;
	bool        d_next_stl_string;
	bool        d_next_qt_string;
	quint32     d_memory_addr;
	quint32     d_memory_size;
	QMutex      d_busy_mutex;

// ******* METHODS *******
public:
	void start( const QString&, const QString& );
	void stop();
private:
	void               request ( const QString& );
	void     callstack_request ();
	void     registers_request ();
	void        disasm_request ();
	void        memory_request ();
	void            after_data ();
	void           customEvent ( QEvent* const );
	void       execute_command ( const QString& );
	qint32             get_pid ( const QString& );
	void       parse_callstack ( const QStringList& );
	void       parse_registers ( const QStringList& );
	void          parse_memory ( const QStringList& );
	void          parse_disasm ( const QStringList& );
	void            set_active ();
	void           parse_scope ( const QStringList& );
	void           parse_value ( const QStringList& );
	void     parse_source_line ( const QString& );
	void          parse_stdout ();
	void     apply_breakpoints ();
	void        set_breakpoint ( const Event* );
	void      unset_breakpoint ( const Event* );
	void         get_item_type ( const Event* );
	void        get_item_value ( const Event* );
	bool  get_std_string_value ( const QStringList&, QString& );
	qint32 get_qt_string_value ( const QStringList&, QString& );
//	void            print_data ( const QStringList& );
//	void         analyze_value ( const QStringList& );
//	void          print_fields ( qint32, const QStringList&, QStringList::const_iterator& );
private slots:
	void           read_stdout ();
	void           read_stderr ();
	void       process_started ();
	void      process_finished ( int );
};

#endif // INCLUDED_Debugger_h
