/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Event.h
 * DATE   : 3.09.2006
 *******************************************************************/
#ifndef INCLUDED_Event_h
#define INCLUDED_Event_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QEvent>
#include <QString>
#include <QVariant>
#include <QVector>

/*------- class declaration:
-------------------------------------------------------------------*/
class Event : public QEvent
{
// ******* TYPES *******
public:
   enum {
      // File ...................................
      OPEN_FILE_REQUEST = ( QEvent::User + 1 ),
      NEW_FILE_REQUEST,
      SAVE_FILE_REQUEST,
      SAVE_FILE_AS_REQUEST,
      SAVE_ALL_FILES_REQUEST,
      PRINT_REQUEST,
      PRINT_FAST_REQUEST,
      PROJECT_SELECTED_REQUEST,
      // Edit ...................................
      UNDO_REQUEST,
      REDO_REQUEST,
      CUT_REQUEST,
      COPY_REQUEST,
      PASTE_REQUEST,
      DELETE_REQUEST,
      SELECT_ALL_REQUEST,
      PROPERTIES_REQUEST,
      // Search .................................
      FIND_REQUEST,
      FIND_NEXT_REQUEST,
      FIND_PREVIOUS_REQUEST,
      REPLACE_REQUEST,
      GOTO_LINE_REQUEST,
      SWITCH_FILES_REQUEST,
      FIND_IN_PROJECT_FILES_REQUEST,
      // Project ................................
      OPEN_PROJECT_REQUEST,
      CLOSE_PROJECT_REQUEST,
      NEW_PROJECT_REQUEST,
      PROJECT_PROPERTIES_REQUEST,
      ADD_FILE_TO_PROJECT_REQUEST,
      REMOVE_FILE_FROM_PROJECT_REQUEST,
      RUN_REQUEST,
      RUN_REQUEST_EXTERNAL,
      BUILD_REQUEST,
      REBUILD_REQUEST,
      BREAK_REQUEST,
      // Debugger ...................................
      DBG_BREAKPOINT_REQUEST,
      DBG_BREAKPOINT_CLEAR_ALL_REQUEST,
      DBG_LIST_BREAKPOINT_REQUEST,
      DBG_RUN_REQUEST,
      DBG_RUN_REQUEST_EXTERNAL,
      DBG_PAUSE_REQUEST,
      DBG_STEP_OVER_REQUEST,
      DBG_STEP_INTO_REQUEST,
      DBG_STEP_OUT_REQUEST,
      DBG_CONTINUE_REQUEST,
      DBG_STOP_REQUEST,
      // Document ...................................
      CLOSE_REQUEST,
      CLOSE_ALL_REQUEST,
      TILE_REQUEST,
      CASCADE_REQUEST,
      DOCUMENT_SELECTED_REQUEST,
      // Projekt
      PROJECT_OPENED,
      PROJECT_CLOSED,
      PROJECT_PARSED,
      PROJECT_REPARSED,
      PROJECT_SRC_FILE_ADDED,
      PROJECT_HDR_FILE_ADDED,
      PROJECT_FRM_FILE_ADDED,
      PROJECT_RSC_FILE_ADDED,
      PROJECT_SRC_ONE_FILE_ADDED,
      PROJECT_HDR_ONE_FILE_ADDED,
      PROJECT_FRM_ONE_FILE_ADDED,
      PROJECT_RSC_ONE_FILE_ADDED,
      PROJECT_ADDED_FINISH,
      FILE_IN_PROJECT_RENAME,
      FILE_IN_PROJECT_RENAMED,
      FILE_IN_PROJECT_REMOVED,
      CLASS_SELECTED,
      MEMBER_SELECTED,
      METHOD_SELECTED,
      // Zdarzenia generowane przez edytor
      FILE_OPENED,
      FILE_CLOSED,
      FILE_SELF_CLOSED,
      ALL_FILES_CLOSED,
      FILE_RENAMED,
      ACTIVE_DOCUMENT_CHANGED,
      CURSOR_POSITION_CHANGED,
      COPY_AVAILABLE,
      REDO_AVAILABLE,
      UNDO_AVAILABLE,
      SEARCH_BROWSER_ITEM,
      SELECTED_FILE_AND_ROW,
      PROCESS_STARTED,
      PROCESS_BROKEN,
      PROCESS_BUILD_DATA,
      PROCESS_BUILD_DATA_NEWLINE,
      PROCESS_MESSAGES_DATA,
      REMOVE_FILE_FROM_PROJECT,
      PARSING_FINISHED,
      NEXT_WINDOW_FOCUS,
      // Zmiana konfiguracji.
      FONT_CHANGED,
      FLTREE_PROP_CHANGED,
      CLTREE_PROP_CHANGED,
      BUILD_TAB_PROP_CHANGED,
      MSG_TAB_PROP_CHANGED,
      SEARCH_TAB_PROP_CHANGED,
      // Bookmarks
      BOOKMARK_NEXT_REQUEST,
      BOOKMARK_PREV_REQUEST,
      BOOKMARK_TOGGLE_REQUEST,
      BOOKMARK_ALL_REQUEST,
      BOOKMARK_CLEAR_ALL_REQUEST,
      // Breakpoint
      BREAKPOINT_SET,
      BREAKPOINT_UNSET,
      // Debugger
      DBG_DISABLE_ALL_REQUEST,
      DBG_DISABLE_RUN_REQUEST,
      DBG_ERROR,
      DBG_STARTED,
      DBG_FINISHED,
      DBG_SCOPE,
      DBG_AFTER_SCOPE,
      DBG_AFTER_DATA,
      DBG_GET_ITEM_VALUE,
      DBG_ITEM_VALUE,
      DBG_GET_ITEM_TYPE,
      DBG_ITEM_TYPE,
      DBG_REGISTERS_START,
      DBG_REGISTERS_STOP,
      DBG_REGISTERS_READ,
      DBG_REGISTERS_DATA,
      DBG_CALLSTACK_START,
      DBG_CALLSTACK_STOP,
      DBG_CALLSTACK_READ,
      DBG_CALLSTACK_DATA,
      DBG_DISASM_START,
      DBG_DISASM_STOP,
      DBG_DISASM_READ,
      DBG_DISASM_DATA,
      DBG_MEMORY_START,
      DBG_MEMORY_STOP,
      DBG_MEMORY_READ,
      DBG_MEMORY_DATA,

      //----------------------------------
   };

public:
   explicit Event( int const in_event_id )
   : QEvent( static_cast<QEvent::Type>( in_event_id ))
   {
      d_data.reserve( 0 );
   }
   explicit Event( const int in_event_id, const QVariant in_val )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 1 );
      d_data << in_val;
   }
   explicit Event( const int in_event_id, const QVariant in_val1, const QVariant in_val2 )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 2 );
      d_data << in_val1 << in_val2;
   }
   explicit Event( const int in_event_id, const QVariant in_val1, const QVariant in_val2, const QVariant in_val3 )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 3 );
      d_data << in_val1 << in_val2 << in_val3;
   }
   explicit Event( const int in_event_id, const QVariant in_val1, const QVariant in_val2, const QVariant in_val3, const QVariant in_val4 )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 4 );
      d_data << in_val1 << in_val2 << in_val3 << in_val4;
   }
public:
   QVector< QVariant > d_data;
private:
   Event( const Event& );
   Event& operator=( const Event& );
};

#endif // INCLUDED_Event_h
