#ifndef INCLUDED_Folder_h
#define INCLUDED_Folder_h

/********************************************************************
 * Copyright (C) 2006, 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Folder.h
 * DATE   : 31.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QWidget>
#include <QTextBlock>
#include "QBTCodeBlockInfo.h"
#include "QBTParserFunDef.h"
#include <vector>
#include <map>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QPainter;
class LineInfo;
class Editor;

/*------- class declaration:
-------------------------------------------------------------------*/
class Folder : public QWidget
{
   Q_OBJECT

typedef std::map<QString,QString> TextPair;

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   Folder( QWidget* = 0 );
private:
   Folder( const Folder& );
   Folder& operator= ( const Folder& );

// ******* CONSTANTS *******
private:
   static const QString FOLD_MARKER;

// ******* MEMBERS *******
private:
   QBTParserFunDef* const           parser_;
   const qint32                     width_;
   bool                             can_parse_;
   bool                             can_update_;
   QString                          plain_text_;
   QBTParserFunDef::CodeBlockVector data_;
   TextPair                         text_pair_;

// ******* METHODS *******
public:
   qint32 width                 () const { return width_; }
   void   update_geometry       ( qint32 );
   void   set_properties        ();
   void   parse                 ( bool = true );
   bool   get_pair_text         ( const QString&, QString&, bool );
private:
   bool   on_screen             ( qint32, qint32, qint32 ) const;
   void   paintEvent            ( QPaintEvent* );
   void   mousePressEvent       ( QMouseEvent* );
   void   mouseDoubleClickEvent ( QMouseEvent* );
   void   draw_opened_begin     ( QPainter&, qint32, qint32, qint32 );
   void   draw_closed_begin     ( QPainter&, qint32, qint32, qint32 );
   void   draw_line             ( QPainter&, qint32, qint32, qint32 );
   void   draw_end              ( QPainter&, qint32, qint32, qint32 );
   bool   this_line             ( qint32, qint32, qint32 ) const;
   void   update_data           ();
   void   set_line_info         ( QTextBlock& ) const;
   bool   search_text_block     ( qint32, QTextBlock& ) const;
   bool   close_opened_block    ( Editor*, QTextBlock&, const QBTCodeBlockInfo& );
   bool   open_closed_block     ( Editor*, QTextBlock& );
   qint32 dwukropek             ( const QString& ) const;
private slots:
   void   parser_finished       ();
};

#endif // INCLUDED_Folder_h
