/********************************************************************
 * Copyright (C) iotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.pl)
 * PROJECT: Cobras
 * FILE   : Points.cpp
 * DATE   : 12.02.2007
 *-------------------------------------------------------------------
 * OPIS   : PURE VIRTUAL
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Points.h"
#include <QVector>

//*******************************************************************
// clear                                                      PUBLIC
//-------------------------------------------------------------------
// 1. Usuniecie z kolekcji WSZYTKICH informacji.
// 2. Usuniecie z kolekcji informacji o wskazanym pliku.
//*******************************************************************
void Points::clear()
{
   collection_.clear();
}
void Points::clear( const QString& in_fpath )
{
   collection_.remove( in_fpath );
}
// end of clear

//*******************************************************************
// add                                                        PUBLIC
//-------------------------------------------------------------------
// Jezeli przyslano punkt o statusie UNSET, zostaje on usuniety z
// kolekcji. W przeciwnym przypadku punkt i jego status sa
// zapamietywane.
//*******************************************************************
void Points::add( const QString& in_fpath, const int in_nr, const PointStatus in_status, const QString& in_text )
{
   if( UNSET == in_status ) {
      if( collection_.contains( in_fpath ) ) {
         collection_[ in_fpath ].remove( in_nr );
         if( collection_[ in_fpath ].empty() ) {
            collection_.remove( in_fpath );
         }
      }
   }
   else {
      collection_[ in_fpath ][ in_nr ] = PointInfo( in_status, in_text );
   }
}
// end of add

//*******************************************************************
// prev                                                       PUBLIC
//*******************************************************************
int Points::prev( const QString& in_fpath, const int in_nr )
{
   if( collection_.contains( in_fpath ) ) {
      QList<int> pts = collection_[ in_fpath ].keys();
      if( !pts.empty() ) {
         qSort( pts.begin(), pts.end(), qGreater<int>() );
         const QList<int>::const_iterator it = qUpperBound( pts.begin(), pts.end(), in_nr, qGreater<int>() );
         if( it != pts.end() ) return *it;
      }
   }
   return -1;
}
// end of prev

//*******************************************************************
// next                                                       PUBLIC
//*******************************************************************
int Points::next( const QString& in_fpath, const int in_nr )
{
   if( collection_.contains( in_fpath ) ) {
      QList<int> pts = collection_[ in_fpath ].keys();
      if( !pts.empty() ) {
         qSort( pts.begin(), pts.end() );
         const QList<int>::const_iterator it = qUpperBound( pts.begin(), pts.end(), in_nr );
         if( it != pts.end() ) return *it;
      }
   }
   return -1;
}
// end of next

//*******************************************************************
// status                                                     PUBLIC
//-------------------------------------------------------------------
// Jezeli wskazany punkt jest w kolekcji zwracany jest jego status.
// W przecienym przypadku zwracany jest 'status' UNKNOWN.
//*******************************************************************
Points::PointStatus Points::status( const QString& in_fpath, const int in_nr )
{
   if( collection_.contains( in_fpath ) ) {
      if( collection_[ in_fpath ].contains( in_nr ) ) {
         return collection_[ in_fpath ][ in_nr ].status();
      }
   }
   return UNKNOWN;
}
// end of status

//*******************************************************************
// file_data                                                  PUBLIC
//-------------------------------------------------------------------
// Zwraca informacje o punktach dla konkretnego pliku.
//*******************************************************************
void Points::file_data ( const QString& in_fpath, QHash<int,PointInfo>& out_data )
{
   out_data.clear();
   if( collection_.contains( in_fpath ) ) {
      out_data = collection_[ in_fpath ];
   }
}
// end of file_data
