/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Points.h
 * DATE   : 12.02.2007
 *******************************************************************/
#ifndef INCLUDED_Points_h
#define INCLUDED_Points_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QObject>
#include <QString>
#include <QVector>
#include <QHash>

/*------- class declaration:
-------------------------------------------------------------------*/
class Points : public QObject
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
protected:
   Points() : QObject() {}
   ~Points() {
      collection_.clear();
   }
private:
   Points( const Points& );
   Points& operator=( const Points& );

// ******* TYPES *******
public:
   typedef enum {
      UNKNOWN = 0,
      UNSET,
      SET,
      DISABLED
   } PointStatus;

   class PointInfo {
      public:
         PointInfo()
         : status_( UNKNOWN )
         , text_  ( QString() )
         {}
         PointInfo( const PointStatus in_status, const QString& in_text )
         : status_( in_status )
         , text_  ( in_text )
         {}
         PointInfo( const PointInfo& in_other )
         {
            status_ = in_other.status_;
            text_   = in_other.text_;
         }
         PointInfo& operator=( const PointInfo& in_other )
         {
            if( this != &in_other ) {
               status_ = in_other.status_;
               text_   = in_other.text_;
            }
            return *this;
         }
      public:
         PointStatus status () const { return status_; }
         const QString& text() const { return text_;   }
      private:
         PointStatus status_;
         QString     text_;
   };

    //                sciezka do pliku
    //                |           numer linii
    //                |           |
   typedef QHash< QString, QHash<int, PointInfo > > Collection;

// ******* MEMBERS *******
private:
   Collection collection_;

// ******* METHODS *******
public:
   void              clear     ();
   void              clear     ( const QString& );
   void              add       ( const QString&, int, PointStatus, const QString& );
   PointStatus       status    ( const QString&, int );
   int               prev      ( const QString&, int );
   int               next      ( const QString&, int );
   const Collection& data      () { return collection_; }
   void              file_data ( const QString&, QHash<int,PointInfo>& );
   
   virtual void change_status ( const QString&, int, const QString& ) = 0;
};

#endif //  INCLUDED_Points_h
