/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Project.h
 * DATE   : 28.09.2006
 *******************************************************************/
#ifndef INCLUDED_Project_h
#define INCLUDED_Project_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_Shared_h
#include "Shared.h"
#endif // INCLUDED_Shared_h

#ifndef INCLUDED_QObject
#define INCLUDED_QObject
#include <QObject>
#endif // INCLUDED_QObject

#ifndef INCLUDED_QHash
#define INCLUDED_QHash
#include <QHash>
#endif // INCLUDED_QHash

#ifndef INCLUDED_QSet
#define INCLUDED_QSet
#include <QSet>
#endif // INCLUDED_QSet

#ifndef INCLUDED_QFileInfo
#define INCLUDED_QFileInfo
#include <QFileInfo>
#endif // INCLUDED_QFileInfo


/*------- forward declarations:
-------------------------------------------------------------------*/
class QTextStream;

/*------- class declaration:
-------------------------------------------------------------------*/
class Project : public QObject
{
	Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
	Project();
private:
	Project( const Project& );
	Project& operator=( const Project& );

// ******* CONSTANTS *******
private:
	static const char* const CAPTION;
	static const char* const CANT_OPEN_FILE;
	static const char* const CANT_SAVE_FILE;
	static const char* const PROJECT_FILE_NOT_FOUND;
public:
	static const char* const QT_CORE;
	static const char* const QT_GUI;
	static const char* const QT_NETWORK;
	static const char* const QT_OPENGL;
	static const char* const QT_SQL;
	static const char* const QT_SVG;
	static const char* const QT_XML;
	static const char* const CFG_QT;
	static const char* const CFG_X11;
	static const char* const CFG_THREAD;
	static const char* const CFG_RELEASE;
	static const char* const CFG_DEBUG;
	static const char* const CFG_RELEASE_DEBUG;
	static const char* const CFG_BUILD_ALL;
	static const char* const CFG_ORDERED;
	static const char* const CFG_WARN_ON;
	static const char* const CFG_WARN_OFF;


// ******* TYPES *******
public:
	typedef QVector< QString >           Vars;
	typedef QSet< QString >              Values;
	typedef QHash< qint32, Values >      MarkValues;
	typedef QHash< QString, MarkValues > Data;
	typedef QHash< QString, QString >    Flags;
	enum {
		MARK_UNKNOWN = 0,
		MARK_EQU,
		MARK_PLUS,
		MARK_MINUS
	};

// ******* MEMBERS *******
private:
	bool      d_is_opened;
	QString   d_buffer;
	QFileInfo d_file_info;
	Vars      d_vars;
	Data      d_data;
	Flags     d_qt_make;
	qint32    d_idx;
	

// ******* METHODS *******
public:
	bool     change_properties ();
	void        get_properties ( QString& out_fpath,
								 QString& out_tmpl,
								 QHash< QString, bool >& out_qt,
								 QHash< QString, bool >& out_cfg );
	bool                create ( QString &out_fpath );
	void        set_properties ( const QString &in_fpath,
								 const QString &in_tmpl,
								 const QHash< QString, bool >& in_qt,
								 const QHash< QString, bool >& in_cfg );
	void         set_var_files ( const QString& in_varname,
								 const QStringList& in_files );
	bool                  open ( const QString& );
	void                 close ();
	bool             is_opened () const;
	const QString&   file_name ();
	const QString&        name ();
	const QString&         dir ();
	const QString&        path ();
	const QString&   prg_fpath ();
	bool                   src ( QStringList& out_src_list );
	bool                   hdr ( QStringList& out_hdr_list );
	bool                    ui ( QStringList& out_ui_list  );
	bool                   qrc ( QStringList& out_qrc_list );
	void         get_var_files ( const QString& in_var_name, QStringList& out_list );
	const QString&    dest_dir ();

	const QString&    add_file ( const QString& in_fpath );		
	const QString& rename_file ( const QString& in_old_fpath, const QString& in_new_fpath );
	const QString& remove_file ( const QString& in_fpath );
	bool                  save ();
private:
	void           set_qt_make ();
	void              get_tmpl ( QString& out_tmpl );
	void                get_qt ( QHash< QString, bool >& out_qt );
	void               get_cfg ( QHash< QString, bool >& out_cfg );
	void               reset ();
	bool       parse_project ( const QString& );
	bool   read_project_file ( const QString& in_fpath, QStringList& out_list );
	void           clip_line ( const QString& in_line, QString& out_varname, qint32& out_mark, QString& out_values ) const;
	bool      normalize_line ( QString& );
	void            save_var ( QTextStream& out_stream, const QString& in_varname );
	bool      is_source_file ( const QString& ) const;	
	bool               empty ( const QString& );
	const QString&  var_name ( const QString& in_fpath );
	void         add_varname ( const QString& in_varname );
	void         del_varname ( const QString& in_varname );
};

inline bool Project::is_opened() const {
	return d_is_opened;
}

#endif // INCLUDED_Project_h
