/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectController.h
 * DATE   : 21.09.2006
 *******************************************************************/
#ifndef INCLUDED_ProjectController_h
#define INCLUDED_ProjectController_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_Project_h
#include "Project.h"
#endif // INCLUDED_Project_h

#include "Debugger.h"

#ifndef INCLUDED_QObject
#define INCLUDED_QObject
#include <QObject>
#endif // INCLUDED_QObject

#ifndef INCLUDED_QFileInfo
#define INCLUDED_QFileInfo
#include <QFileInfo>
#endif // INCLUDED_QFileInfo

#ifndef INCLUDED_QStringList
#define INCLUDED_QStringList
#include <QStringList>
#endif // INCLUDED_QStringList


/*------- forward declaration:
-------------------------------------------------------------------*/
class Event;
class Process;

/*------- class declaration:
-------------------------------------------------------------------*/
class ProjectController : public QObject
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   ProjectController( QWidget* );
   ~ProjectController();
private:
   ProjectController( const ProjectController& );
   ProjectController& operator=( const ProjectController& );

// ******* TYPES *******
private:
	enum {
		UNKNOWN = 0,
		SOURCES,
		HEADERS,
		FORMS,
		RESOURCES
	};
// ******* CONSTANTS *******
private:
	static const char* const FIND_CAPTION;
	static const char* const DIALOG_OPEN_PROJECT_CAPTION;
	static const char* const PROJECTS_FILTER;
	static const char* const CAPTION;
	static const char* const CANT_OPEN_FILE;
	static const char* const NOT_VALID_PROJECT;
	static const char* const PROJECT_IS_OPENED;
	static const char* const ADD_FILE_TO_PROJECT;
	static const char* const UNKNOWN_FILE_TYPE;
	static const char* const INVALID_EXTENSION;
	static const char* const DIR_NOT_EXISTS;
	static const char* const CANT_CREATE;
	static const char* const ALREADY_EXISTS;
	static const char* const CANT_APPEND;
	static const char* const FILE_TYPE_CHANGED;
	static const char* const RENAME_PROMPT;

// ******* MEMBERS *******
private:
	Project           d_project;
	QWidget*    const d_parent;
	QString           d_last_prj_dir;
	Process*          d_process;
	Debugger*         d_debugger;
	QStringList       d_files;

// ******* METHODS *******
private:
	void              customEvent ( QEvent* );
	void       project_properties ();
	void                   rename ( const QString& );
	void             open_project ();
	void         project_selected ( Event* );
	bool                 can_open ();
	void                     open ( const QString& );
	void            close_project ();
	void              new_project ();
	void remove_file_from_project ( Event* );
	void                add_items ( const QStringList&, const int );
	void      add_file_to_project ();
	bool                 add_file ( const QString& );
	void         update_file_view ( const QString&, const QString& );
	void    find_in_project_files ();
	void            find_in_files ( const QRegExp& );
	void        find_in_this_file ( const QString&, const QRegExp& );
	void             before_qmake ( QStringList& );
	void              after_qmake ( QStringList& );
	void                      run ();
	void                    build ();
	void                  rebuild ();
	void            process_break ();
	void           process_broken ();
	void                  dbg_run ();
	void                 dbg_stop ();
public:
	const QString&    project_dir ();
};

inline const QString& ProjectController::project_dir() {
	return d_project.dir();
}

#endif // INCLUDED_ProjectController_h
