/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectPropGeneral.h
 * DATE   : 12.12.2006
 *******************************************************************/
#ifndef INCLUDED_ProjectPropGeneral_h
#define INCLUDED_ProjectPropGeneral_h

/*------- include files:
-------------------------------------------------------------------*/
#include "Project.h"
#include <QWidget>
#include <QHash>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QComboBox;
class QCheckBox;
class QRadioButton;
class QLineEdit;
class QPushButton;
class QAbstractButton;

/*------- include files:
-------------------------------------------------------------------*/
class ProjectPropGeneral : public QWidget
{
	Q_OBJECT
public:
	ProjectPropGeneral( QWidget* in_parent = 0 );
private:
	ProjectPropGeneral( const ProjectPropGeneral& );
	ProjectPropGeneral& operator=( const ProjectPropGeneral& );

// ******* CONSTANTS *******
private:
	static const char* const TEMPLATE;
	static const char* const PATH;
	static const char* const CAPTION;
	static const char* const FILTER;
	static const char* const FILE_EXIST_CAPTION;
	static const char* const FILE_EXIST_MSG;

// ******* MEMBERS *******
private:
	QLineEdit*    const d_project_path_le;
	QPushButton*  d_project_select_pbtn;
	QComboBox*    d_template_cbx;
	QCheckBox*    d_core_cbx;
	QCheckBox*    d_gui_cbx;
	QCheckBox*    d_network_cbx;
	QCheckBox*    d_opengl_cbx;
	QCheckBox*    d_sql_cbx;
	QCheckBox*    d_svg_cbx;
	QCheckBox*    d_xml_cbx;
	QCheckBox*    d_qt_cbx;
	QCheckBox*    d_x11_cbx;
	QCheckBox*    d_thread_cbx;
	QRadioButton* d_release_rbtn;
	QRadioButton* d_debug_rbtn;
	QRadioButton* d_release_and_debug_rbtn;
	QCheckBox*    d_build_all_cbx;
	QCheckBox*    d_ordered_cbx;
	QRadioButton* d_warn_on_rbtn;
	QRadioButton* d_warn_off_rbtn;

	QString       d_fpath;
	QString       d_tmpl;
	QHash< QString, bool > d_qt;
	QHash< QString, bool > d_cfg;

// ******* METHODS *******
public:
	bool accept();
	void get_data ( QString& out_fpath,
					QString& out_tmpl,
					QHash< QString, bool > &out_qt,
					QHash< QString, bool > &out_cfg );
	void set_data ( const QString& in_fpath,
					const QString& in_tmpl,
					const QHash< QString, bool > &in_qt,
					const QHash< QString, bool > &in_cfg );
	const QString& dir();
private:
	bool    create_project_file ( const QString& in_fpath );
	void           set_defaults ();
private slots:
	void       template_changed ( const QString& in_text );
	void release_and_debug_slot ( QAbstractButton* in_button );
	void          get_path_slot ();
	void      text_changed_slot ( const QString& in_text );
signals:
	void enable_tabs( bool );
};

#endif // INCLUDED_ProjectPropGeneral_h
