/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesEdit.h
 * DATE   : 22.10.2006
 *******************************************************************/
#ifndef INCLUDED_PropertiesEdit_h
#define INCLUDED_PropertiesEdit_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_Config_h
#include "Config.h"
#endif // INCLUDED_Config_h

#ifndef INCLUDED_QWidget
#define INCLUDED_QWidget
#include <QWidget>
#endif // INCLUDED_QWidget

#ifndef INCLUDED_QVariant
#define INCLUDED_QVariant
#include <QVariant>
#endif // INCLUDED_QVariant

#ifndef INCLUDED_QList
#define INCLUDED_QList
#include <QList>
#endif // INCLUDED_QList

/*------- forward declarations:
-------------------------------------------------------------------*/
class QVBoxLayout;
class QHBoxLayout;
class QCheckBox;
class QGroupBox;
class QGridLayout;
class QLabel;
class QPushButton;
class QListWidget;
class QSpinBox;
class QLineEdit;

/*------- class declaration:
-------------------------------------------------------------------*/
class PropertiesEdit : public QWidget
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION ********
public:
   PropertiesEdit( QWidget* );
private:
   PropertiesEdit( const PropertiesEdit& );
   PropertiesEdit& operator=( const PropertiesEdit& );

// ******* CONSTANTS *******
private:
   static const char* const DEFAULT;
   static const char* const HILI_LINE;
   static const char* const TAB_WIDTH;
   static const char* const AUTO_INDENT;
   static const char* const TAB2SPC;
   static const char* const NUMBERS;
   static const char* const RMARGIN;
   static const char* const EDITING_GRP;
   static const char* const BACKGROUNDS;
   static const char* const CPP_BACKGROUND;
   static const char* const HPP_BACKGROUND;
   static const char* const LINE_BACKGROUND;
   static const char* const SYNTAX_GRP;
   static const char* const SYNTAX_CBX;
   static const char* const EXAMPLE_LBL;
   static const char* const FONT;
   static const char* const COLOR;
   static const char* const ITALIC;
   static const char* const BOLD;
   static const char* const LIST_ITEMS[ Config::EndMarker ];

// ******* MEMBERS *******
private:
   QVBoxLayout* const d_main_layout;
   QCheckBox*   const d_cbx_default;
   QGroupBox*   const d_gbox1;
   QGridLayout* const d_grid;
   QLabel*      const d_hili_line_lbl;
   QCheckBox*   const d_hili_line_cbx;
   QLabel*      const d_tab_width_lbl;
   QSpinBox*    const d_tab_width_spx;
   QLabel*      const d_auto_indent_lbl;
   QCheckBox*   const d_auto_indent_cbx;
   QLabel*      const tab2spc_lbl_;
   QCheckBox*   const tab2spc_cbx_;
   QPushButton* const numbers_btn_;
   QPushButton* const rmargin_btn_;
   QGroupBox*   const d_gbox2;
   QGridLayout* const d_bkglay;
   QLineEdit*   const d_cpp_view;
   QLineEdit*   const d_hpp_view;
   QLineEdit*   const d_currline_view;
   QPushButton* const d_cpp_bkg_btn;
   QPushButton* const d_hpp_bkg_btn;
   QPushButton* const d_currline_btn;
   QCheckBox*   const d_cbx_syntax;
   QGroupBox*   const d_gbox3;
   QHBoxLayout* const d_syntax_layout;
   QVBoxLayout* const d_lstlay;
   QListWidget* const d_list;
   QLabel*      const d_example_lbl;
   QVBoxLayout* const d_sblay;
   QPushButton* const d_btn_font;
   QPushButton* const d_btn_color;
   QCheckBox*   const d_cbx_italic;
   QCheckBox*   const d_cbx_bold;

   bool               d_showed;
   QFont              d_font;
   QRgb               d_cpp_background;
   QRgb               d_hpp_background;
   QRgb               d_currline_background;
   QVector< Config::SyntaxInfo > d_syntax;
   
// ******* METHODS *******
private:
   void       showEvent ( QShowEvent* );
   void set_example_lbl ( const QFont&, QRgb );
   void  set_background ( QLineEdit*, QRgb );
private slots:
   void row_changed     ( int );
   void default_clicked  ();
   void cpp_bkg_color    ();
   void hpp_bkg_color    ();
   void currline_color   ();
   void font_clicked     ();
   void color_clicked    ();
   void italic_clicked   ();
   void bold_clicked     ();
   void save_data        ();
   void numbers_selected ();
   void rmargin_selected ();
};

#endif // INCLUDED_PropertiesEdit_h
