/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesNumbers.cpp
 * DATE   : 12.05.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "PropertiesNumbers.h"
#include "Config.h"
#include "EventsController.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QCheckBox>
#include <QGroupBox>
#include <QPushButton>
#include <QLineEdit>
#include <QColorDialog>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const PropertiesNumbers::CAPTION      = QT_TR_NOOP( "Line Numbers" );
const char* const PropertiesNumbers::USE_DEFAULTS = QT_TR_NOOP( "&Use defaults" );
const char* const PropertiesNumbers::SETTINGS     = QT_TR_NOOP( "Color Settings" );
const char* const PropertiesNumbers::DISPLAY      = QT_TR_NOOP( "&Display numbers" );
const char* const PropertiesNumbers::FONT_COLOR   = QT_TR_NOOP( "&Font" );
const char* const PropertiesNumbers::BKG_COLOR    = QT_TR_NOOP( "&Background" );
const char* const PropertiesNumbers::SEL_COLOR    = QT_TR_NOOP( "&Selected background" );
const char* const PropertiesNumbers::SET_DEFAULTS = QT_TR_NOOP( "Set de&faults" );
const char* const PropertiesNumbers::ACCEPT       = QT_TR_NOOP( "&Accept" );
const char* const PropertiesNumbers::EXIT         = QT_TR_NOOP( "&Exit" );

//*******************************************************************
// PropertiesNumbers                                     CONSTRUCTOR
//*******************************************************************
PropertiesNumbers::PropertiesNumbers( QWidget* const in_parent )
: QDialog           ( in_parent )
, gbox_             ( new QGroupBox( tr( SETTINGS     ), this ) )
, use_defaults_cbx_ ( new QCheckBox( tr( USE_DEFAULTS ), this ) )
, display_cbx_      ( new QCheckBox( tr( DISPLAY      ), this ) )
, demo_font_color_  ( new QLineEdit( gbox_ ) )
, demo_bkg_color_   ( new QLineEdit( gbox_ ) )
, demo_sel_color_   ( new QLineEdit( gbox_ ) )
, font_color_btn_   ( new QPushButton( QIcon( ":/img/colors.png" ), tr( FONT_COLOR ), gbox_ ) )
, bkg_color_btn_    ( new QPushButton( QIcon( ":/img/colors.png" ), tr( BKG_COLOR  ), gbox_ ) )
, sel_color_btn_    ( new QPushButton( QIcon( ":/img/colors.png" ), tr( SEL_COLOR  ), gbox_ ) )
, set_defaults_btn_ ( new QPushButton( tr( SET_DEFAULTS ), this ) )
, accept_btn_       ( new QPushButton( tr( ACCEPT       ), this ) )
, exit_btn_         ( new QPushButton( tr( EXIT         ), this ) )
{
	setWindowTitle( tr( CAPTION ) );

	QVBoxLayout* const mlayout    = new QVBoxLayout( this );
	QVBoxLayout* const boxlayout  = new QVBoxLayout( gbox_ );
	QGridLayout* const gridlayout = new QGridLayout;
	QHBoxLayout* const btnlayout  = new QHBoxLayout;
	
	demo_font_color_->setMaximumHeight( 12 );
	demo_bkg_color_->setMaximumHeight( 12 );
	demo_sel_color_->setMaximumHeight( 12 );

	gridlayout->addWidget( font_color_btn_ , 0, 0 );
	gridlayout->addWidget( bkg_color_btn_  , 1, 0 );
	gridlayout->addWidget( sel_color_btn_  , 2, 0 );
	gridlayout->addWidget( demo_font_color_, 0, 1 );
	gridlayout->addWidget( demo_bkg_color_ , 1, 1 );
	gridlayout->addWidget( demo_sel_color_ , 2, 1 );
	boxlayout->addLayout( gridlayout );

	btnlayout->addWidget( set_defaults_btn_ );
	btnlayout->addStretch();
	btnlayout->addWidget( accept_btn_ );
	btnlayout->addWidget( exit_btn_ );

	mlayout->addWidget( use_defaults_cbx_ );
	mlayout->addWidget( display_cbx_ );
	mlayout->addWidget( gbox_ );
	mlayout->addLayout( btnlayout );

	connect(	use_defaults_cbx_, SIGNAL( clicked             ( bool ) ),
				this             , SLOT  ( use_defaults_clicked( bool ) ) );
	connect( font_color_btn_  , SIGNAL( clicked             () ),
				this             , SLOT  ( font_color_clicked  () ) );
	connect( bkg_color_btn_   , SIGNAL( clicked             () ),
				this             , SLOT  ( bkg_color_clicked   () ) );
	connect( sel_color_btn_   , SIGNAL( clicked             () ),
				this             , SLOT  ( sel_color_clicked   () ) );
	connect( set_defaults_btn_, SIGNAL( clicked             () ),
				this             , SLOT  ( set_defaults_clicked() ) );
	connect( accept_btn_      , SIGNAL( clicked             () ),
				this             , SLOT  ( accept_settings     () ) );
	connect( exit_btn_        , SIGNAL( clicked             () ),
				this             , SLOT  ( reject              () ) );

	// Odczyt danych z Config i stosowne ustawienie widgetow.
	use_defaults_cbx_->setChecked   ( Config::instance()->margin_get_use_defaults()       );
	display_cbx_->setChecked        ( Config::instance()->margin_get_use_numbers ( true ) );
	set_demo_color( demo_font_color_, Config::instance()->margin_get_font_color  ( true ) );
	set_demo_color( demo_bkg_color_ , Config::instance()->margin_get_bkg_color   ( true ) );
	set_demo_color( demo_sel_color_ , Config::instance()->margin_get_sel_color   ( true ) );
	
	// Aktywacja/Dezaktywacja widgetow.
	use_defaults_clicked( Config::instance()->margin_get_use_defaults() );
}
// end of PropertiesNumbers

//*******************************************************************
// ~PropertiesNumbers                                     DESTRUCTOR
//*******************************************************************
PropertiesNumbers::~PropertiesNumbers()
{
}
// end of ~PropertiesNumbers

//*******************************************************************
// set_demo_color                                            PRIVATE
//*******************************************************************
void PropertiesNumbers::set_demo_color( QLineEdit* const in_widget, const QRgb in_rgb )
{
	QPalette palette;
	QColor color;
	color.setRgba( in_rgb );
	palette.setColor( in_widget->backgroundRole(), color );
	in_widget->setPalette( palette );
}
// end of set_demo_color

//*******************************************************************
// get_demo_color                                            PRIVATE
//*******************************************************************
QRgb PropertiesNumbers::get_demo_color( const QLineEdit* const in_widget )
{
	return in_widget->palette().color( in_widget->backgroundRole() ).rgb();
}
// end of get_demo_color

//*******************************************************************
// accept_settings                                      PRIVATE slot
//*******************************************************************
void PropertiesNumbers::accept_settings()
{
	Config::instance()->margin_set_use_defaults( use_defaults_cbx_->isChecked()     );
	Config::instance()->margin_set_use_numbers ( display_cbx_->isChecked     ()     );
	Config::instance()->margin_set_font_color  ( get_demo_color( demo_font_color_ ) );
	Config::instance()->margin_set_bkg_color   ( get_demo_color( demo_bkg_color_  ) );
	Config::instance()->margin_set_sel_color   ( get_demo_color( demo_sel_color_  ) );

	Config::instance()->save();
	EventsController::instance()->send_event( Event::FONT_CHANGED );
}
// end of accept_settings

//*******************************************************************
// set_defaults_clicked                                 PRIVATE slot
//*******************************************************************
void PropertiesNumbers::set_defaults_clicked()
{
	const bool prv_stat = Config::instance()->margin_get_use_defaults();
	use_defaults_cbx_->setChecked( prv_stat );

	Config::instance()->margin_set_use_defaults( true );
	display_cbx_->setChecked( Config::instance()->margin_get_use_numbers() );
	set_demo_color( demo_font_color_, Config::instance()->margin_get_font_color() );
	set_demo_color( demo_bkg_color_ , Config::instance()->margin_get_bkg_color () );
	set_demo_color( demo_sel_color_ , Config::instance()->margin_get_sel_color () );
	Config::instance()->margin_set_use_defaults( prv_stat );	
}
// end of set_defaults_clicked

//*******************************************************************
// use_defaults_clicked                                 PRIVATE slot
//*******************************************************************
void PropertiesNumbers::use_defaults_clicked( const bool in_flag )
{
	display_cbx_->setEnabled( !in_flag );
	gbox_->setEnabled( !in_flag );
	set_defaults_btn_->setEnabled( !in_flag );
}
// end of use_defaults_clicked

//*******************************************************************
// font_color_clicked                                   PRIVATE slot
//*******************************************************************
void PropertiesNumbers::font_color_clicked()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( get_demo_color( demo_font_color_ ), &ok, this );
	if( ok ) {
		set_demo_color( demo_font_color_, qrgb );
	}
}
// end of font_color_clicked

//*******************************************************************
// bkg_color_clicked                                    PRIVATE slot
//*******************************************************************
void PropertiesNumbers::bkg_color_clicked()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( get_demo_color( demo_bkg_color_ ), &ok, this );
	if( ok ) {
		set_demo_color( demo_bkg_color_, qrgb );
	}
}
// end of bkg_color_clicked

//*******************************************************************
// sel_color_clicked                                    PRIVATE slot
//*******************************************************************
void PropertiesNumbers::sel_color_clicked()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( get_demo_color( demo_sel_color_ ), &ok, this );
	if( ok ) {
		set_demo_color( demo_sel_color_, qrgb );
	}
}
// end of sel_color_clicked
