/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesRMargin.cpp
 * DATE   : 12.05.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "PropertiesRMargin.h"
#include "Config.h"
#include "EventsController.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QCheckBox>
#include <QGroupBox>
#include <QPushButton>
#include <QLineEdit>
#include <QColorDialog>
#include <QSpinBox>
#include <QLabel>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const PropertiesRMargin::CAPTION       = QT_TR_NOOP( "Right Margin Line" );
const char* const PropertiesRMargin::USE_DEFAULTS  = QT_TR_NOOP( "&Use defaults" );
const char* const PropertiesRMargin::SETTINGS      = QT_TR_NOOP( "Settings" );
const char* const PropertiesRMargin::DISPLAY       = QT_TR_NOOP( "&Display margin" );
const char* const PropertiesRMargin::LINE_POSITION = QT_TR_NOOP( "Line &position (in characters)" );
const char* const PropertiesRMargin::LINE_COLOR    = QT_TR_NOOP( "Line &color" );
const char* const PropertiesRMargin::SET_DEFAULTS  = QT_TR_NOOP( "Set de&faults" );
const char* const PropertiesRMargin::ACCEPT        = QT_TR_NOOP( "&Accept" );
const char* const PropertiesRMargin::EXIT          = QT_TR_NOOP( "&Exit" );

//*******************************************************************
// PropertiesRMargin                                     CONSTRUCTOR
//*******************************************************************
PropertiesRMargin::PropertiesRMargin( QWidget* const in_parent )
: QDialog( in_parent )
, gbox_             ( new QGroupBox( tr( SETTINGS     ), this ) )
, use_defaults_cbx_ ( new QCheckBox( tr( USE_DEFAULTS ), this ) )
, n_spin_           ( new QSpinBox ( gbox_ ) )
, display_cbx_      ( new QCheckBox( tr( DISPLAY      ), this ) )
, demo_line_color_  ( new QLineEdit( gbox_ ) )
, line_color_btn_   ( new QPushButton( QIcon( ":/img/colors.png" ), tr( LINE_COLOR ), gbox_ ) )
, set_defaults_btn_ ( new QPushButton( tr( SET_DEFAULTS ), this ) )
, accept_btn_       ( new QPushButton( tr( ACCEPT       ), this ) )
, exit_btn_         ( new QPushButton( tr( EXIT         ), this ) )
{
	setWindowTitle( tr( CAPTION ) );

	QVBoxLayout* const mlayout    = new QVBoxLayout( this );
	QVBoxLayout* const boxlayout  = new QVBoxLayout( gbox_ );
	QGridLayout* const gridlayout = new QGridLayout;
	QHBoxLayout* const btnlayout  = new QHBoxLayout;
	
	demo_line_color_->setMaximumHeight( 12 );

	n_spin_->setMaximumHeight( fontMetrics().height() + 4 );
	n_spin_->setMinimum( 20 );
	n_spin_->setMaximum( 200 );

	QLabel* const lbl = new QLabel( tr( LINE_POSITION ), gbox_ );
	lbl->setBuddy( n_spin_ );
	
	gridlayout->addWidget( lbl             , 0, 0 );
	gridlayout->addWidget( line_color_btn_ , 1, 0 );
	gridlayout->addWidget( n_spin_         , 0, 1 );
	gridlayout->addWidget( demo_line_color_, 1, 1 );
	boxlayout->addLayout( gridlayout );

	btnlayout->addWidget( set_defaults_btn_ );
	btnlayout->addStretch();
	btnlayout->addWidget( accept_btn_ );
	btnlayout->addWidget( exit_btn_ );

	mlayout->addWidget( use_defaults_cbx_ );
	mlayout->addWidget( display_cbx_ );
	mlayout->addWidget( gbox_ );
	mlayout->addLayout( btnlayout );

	connect(	use_defaults_cbx_, SIGNAL( clicked             ( bool ) ),
				this             , SLOT  ( use_defaults_clicked( bool ) ) );
	connect( line_color_btn_  , SIGNAL( clicked             () ),
				this             , SLOT  ( line_color_clicked  () ) );
	connect( set_defaults_btn_, SIGNAL( clicked             () ),
				this             , SLOT  ( set_defaults_clicked() ) );
	connect( accept_btn_      , SIGNAL( clicked             () ),
				this             , SLOT  ( accept_settings     () ) );
	connect( exit_btn_        , SIGNAL( clicked             () ),
				this             , SLOT  ( reject              () ) );

	// Odczyt danych z Config i stosowne ustawienie widgetow.
	use_defaults_cbx_->setChecked   ( Config::instance()->rmargin_get_use_defaults()       );
	display_cbx_->setChecked        ( Config::instance()->rmargin_get_use         ( true ) );
	n_spin_->setValue               ( Config::instance()->rmargin_get_n           ( true ) );
	set_demo_color( demo_line_color_, Config::instance()->rmargin_get_color       ( true ) );
	
	// Aktywacja/Dezaktywacja widgetow.
	use_defaults_clicked( Config::instance()->rmargin_get_use_defaults() );
}
// end of PropertiesRMargin

//*******************************************************************
// ~PropertiesRMargin                                     DESTRUCTOR
//*******************************************************************
PropertiesRMargin::~PropertiesRMargin()
{
}
// end of ~PropertiesRMargin

//*******************************************************************
// set_demo_color                                            PRIVATE
//*******************************************************************
void PropertiesRMargin::set_demo_color( QLineEdit* const in_widget, const QRgb in_rgb )
{
	QPalette palette;
	QColor color;
	color.setRgba( in_rgb );
	palette.setColor( in_widget->backgroundRole(), color );
	in_widget->setPalette( palette );
}
// end of set_demo_color

//*******************************************************************
// get_demo_color                                            PRIVATE
//*******************************************************************
QRgb PropertiesRMargin::get_demo_color( const QLineEdit* const in_widget )
{
	return in_widget->palette().color( in_widget->backgroundRole() ).rgb();
}
// end of get_demo_color

//*******************************************************************
// accept_settings                                      PRIVATE slot
//*******************************************************************
void PropertiesRMargin::accept_settings()
{
	Config::instance()->rmargin_set_use_defaults( use_defaults_cbx_->isChecked()     );
	Config::instance()->rmargin_set_use         ( display_cbx_->isChecked     ()     );
	Config::instance()->rmargin_set_n           ( n_spin_->value()                   );
	Config::instance()->rmargin_set_color       ( get_demo_color( demo_line_color_ ) );

	Config::instance()->save();
	EventsController::instance()->send_event( Event::FONT_CHANGED );
}
// end of accept_settings

//*******************************************************************
// set_defaults_clicked                                 PRIVATE slot
//*******************************************************************
void PropertiesRMargin::set_defaults_clicked()
{
	const bool prv_stat = Config::instance()->rmargin_get_use_defaults();
	use_defaults_cbx_->setChecked( prv_stat );

	Config::instance()->rmargin_set_use_defaults( true );
	display_cbx_->setChecked( Config::instance()->rmargin_get_use() );
	n_spin_->setValue( Config::instance()->rmargin_get_n() );
	set_demo_color( demo_line_color_, Config::instance()->rmargin_get_color() );
	Config::instance()->rmargin_set_use_defaults( prv_stat );	
}
// end of set_defaults_clicked

//*******************************************************************
// use_defaults_clicked                                 PRIVATE slot
//*******************************************************************
void PropertiesRMargin::use_defaults_clicked( const bool in_flag )
{
	display_cbx_->setEnabled( !in_flag );
	gbox_->setEnabled( !in_flag );
	set_defaults_btn_->setEnabled( !in_flag );
}
// end of use_defaults_clicked

//*******************************************************************
// line_color_clicked                                   PRIVATE slot
//*******************************************************************
void PropertiesRMargin::line_color_clicked()
{
	bool ok;
	const QRgb qrgb = QColorDialog::getRgba( get_demo_color( demo_line_color_ ), &ok, this );
	if( ok ) {
		set_demo_color( demo_line_color_, qrgb );
	}
}
// end of line_color_clicked
