/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtMessageTab.cpp
 * DATE   : 3.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtMessageTab.h"
#include "BrowserBuild.h"
#include "BrowserMessages.h"
#include "BrowserSearch.h"
#include "BrowserDbg.h"
#include "EventsController.h"
#include <QLayout>
#include <QTabBar>
#include <QStackedWidget>
#include <QTextBrowser>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const QBtMessageTab::BUILD_TAB_LABEL  = QT_TR_NOOP( "Build"    );
const char* const QBtMessageTab::MSG_TAB_LABEL    = QT_TR_NOOP( "Messages" );
const char* const QBtMessageTab::SEARCH_TAB_LABEL = QT_TR_NOOP( "Search"   );
const char* const QBtMessageTab::DBG_TAB_LABEL    = QT_TR_NOOP( "Debugger" );

//*******************************************************************
// QBtMessageTab                                            CONSTRUCTOR
//*******************************************************************
QBtMessageTab::QBtMessageTab( QWidget* const in_parent ) : QWidget( in_parent )
, tab_bar_        ( new QTabBar        )
, widget_stack_   ( new QStackedWidget )
, build_tab_idx_  ( tab_bar_->addTab( tr( BUILD_TAB_LABEL  ) ) )
, msg_tab_idx_    ( tab_bar_->addTab( tr( MSG_TAB_LABEL    ) ) )
, search_tab_idx_ ( tab_bar_->addTab( tr( SEARCH_TAB_LABEL ) ) )
, dbg_tab_idx_    ( tab_bar_->addTab( tr( DBG_TAB_LABEL    ) ) )
, dbg_browser_    ( new BrowserDbg      )
, msg_browser_    ( new BrowserMessages )
{
   dbg_browser_->setEnabled( false );

   widget_stack_->insertWidget( build_tab_idx_ , new BrowserBuild  );
   widget_stack_->insertWidget( msg_tab_idx_   , msg_browser_      );
   widget_stack_->insertWidget( search_tab_idx_, new BrowserSearch );
   widget_stack_->insertWidget( dbg_tab_idx_   , dbg_browser_      );

   QVBoxLayout* const layout = new QVBoxLayout;   
   layout->addWidget( tab_bar_      );
   layout->addWidget( widget_stack_ );
   setLayout( layout );

   EventsController::instance()->append( this, Event::FIND_IN_PROJECT_FILES_REQUEST );
   EventsController::instance()->append( this, Event::PROCESS_STARTED               );
   EventsController::instance()->append( this, Event::PROCESS_BROKEN                );
   EventsController::instance()->append( this, Event::DBG_RUN_REQUEST               );
   EventsController::instance()->append( this, Event::DBG_FINISHED                  );

   connect( tab_bar_, SIGNAL( currentChanged( int ) ), this, SLOT( tab_changed( int ) ) );
}
// end of QBtMessageTab

//*******************************************************************
// ~QBtMessageTab                                            DESTRUCTOR
//*******************************************************************
QBtMessageTab::~QBtMessageTab()
{
   EventsController::instance()->remove( this );
}
// end of ~QBtMessageTab

//*******************************************************************
// translate                                                  PUBLIC
//*******************************************************************
void QBtMessageTab::retranslate()
{
   tab_bar_->setTabText( build_tab_idx_ , tr( BUILD_TAB_LABEL  ) );
   tab_bar_->setTabText( msg_tab_idx_   , tr( MSG_TAB_LABEL    ) );
   tab_bar_->setTabText( search_tab_idx_, tr( SEARCH_TAB_LABEL ) );
   tab_bar_->setTabText( dbg_tab_idx_   , tr( DBG_TAB_LABEL    ) );
}
// end of translate

//*******************************************************************
// tab_changed                                          PRIVATE slot
//*******************************************************************
void QBtMessageTab::tab_changed( const int in_tab_idx )
{
   widget_stack_->setCurrentIndex( in_tab_idx );
}
// end of tab_changed

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtMessageTab::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast<int>( event->type() );
         
   switch( type ) {
      case Event::FIND_IN_PROJECT_FILES_REQUEST:
         tab_bar_->setCurrentIndex( search_tab_idx_ );
         break;
      case Event::PROCESS_STARTED:
         tab_bar_->setCurrentIndex( build_tab_idx_ );
         break;
      case Event::PROCESS_BROKEN:
         if( msg_browser_->count() > 0 ) {
            tab_bar_->setCurrentIndex( msg_tab_idx_ );
         }
         break;
      case Event::DBG_RUN_REQUEST:
         dbg_browser_->setEnabled( true );         
         tab_bar_->setCurrentIndex( dbg_tab_idx_ );
         break;
      case Event::DBG_FINISHED:
         dbg_browser_->setEnabled( false );
         tab_bar_->setCurrentIndex( build_tab_idx_ );
   }
}
// end of customEvent
