#ifndef INCLUDED_QBtPrinter_h
#define INCLUDED_QBtPrinter_h

/*******************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtPrinter.h
 * DATE   : 2007.12.14
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QWidget>
#include <QFont>
#include <QPainter>
#include <QPrinter>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QTextEdit;
class QPainter;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtPrinter : public QObject
{
   Q_OBJECT

//******* CONSTRUCTION/DESTRUCTION *******
private:
   QBtPrinter();
   ~QBtPrinter();
   QBtPrinter( const QBtPrinter& );
   QBtPrinter& operator=( const QBtPrinter& );

//******* CONSTANTS *******
private:
   static const qint32      DEFAULT_LEFT_MARGIN_OFFSET;
   static const char* const FILE;
   static const char* const PAGE;
   static const char* const CREATOR;

//******* MEMEBERS *******
private:
   QPrinter printer_;
   QPainter painter_;
   QFont    font_;
   QString  file_path_;
   qint32   page_x_;
   qint32   page_y_;
   qint32   page_dx_;
   qint32   page_dy_;
   qint32   x_;
   qint32   y_;
   qint32   left_margin_offset_;
   qint32   org_margin_;

//******* METHODS *******
public:
   static QBtPrinter* instance();
   void print( const QString&, const QTextEdit*, const QFont&, bool );
private:
   void        paginate     ( QStringList&, const QFontMetrics&, bool = false );
   QStringList clip_line    ( const QString&, const QFontMetrics& ) const;
   void        init         ();
   qint32      header_height();
   void        print_header ();
   void        print_footer ( qint32, qint32 );
   qint32      print_text   ( const QString&, const QFontMetrics&, qint32 = Qt::AlignLeft );
   void        trimm_end    ( QString& ) const;
   qint32      print_line   ( const QFontMetrics& );
// void        print_config () const;
};

#endif // INCLUDED_QBtPrinter_h
