/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : TextEdit.cpp
 * DATE   : 12.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "TextEdit.h"
#include "Editor.h"
#include "Margin.h"
#include "Config.h"
#include <QPainter>
#include <QPen>
#include <QTextBlock>
#include <QTextLayout>
#include <QScrollBar>
#include <QtDebug>

//*******************************************************************
// TextEdit                                              CONSTRUCTOR
//*******************************************************************
TextEdit::TextEdit( QWidget* const in_parent )
: QTextEdit( in_parent )
{}
// end of TextEdit

//*******************************************************************
// paintEvent                                      PRIVATE inherited
//*******************************************************************
void TextEdit::paintEvent( QPaintEvent* const in_event )
{
   const Config* const cfg = Config::instance();
   QPainter p( viewport() );

   if( cfg->line_highlight() ) {
      const QTextBlock block = textCursor().block();
      const int y_top        = verticalScrollBar()->value();
      const int block_yt     = qRound( block.layout()->position().y() );
      const int block_height = block.layout()->boundingRect().toRect().height();
         
      QRect rect( 0, block_yt - y_top + 1, viewport()->width(), block_height );
      p.fillRect( rect, QBrush( cfg->currline_background() ) );
   }
   
   if( cfg->rmargin_get_use() ) {
      const int vp_height = viewport()->height();
      const int xpos = cfg->rmargin_get_n() * fontMetrics().width( "X" );
      const QPen cr_pen = p.pen();
      p.setPen( QColor( cfg->rmargin_get_color() ) );
      p.drawLine( xpos, 0, xpos, vp_height - 1 );   
      p.setPen( cr_pen );
   }
    p.end();
   QTextEdit::paintEvent( in_event );   
}
// end of paintEvent
