/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Translator.cpp
 * DATE   : 30.07.2007
 *-------------------------------------------------------------------
 * Translator (singleton) ma za zadanie obslugiwac wszystkie sprawy
 * zwiazane ze tlumaczeniami napisow na inne jezyki.
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Translator.h"
#include "Settings.h"
#include <QApplication>
#include <QtDebug>

/*------- local variables:
-------------------------------------------------------------------*/
Translator* Translator::instance_ = 0;

/*------- loacal constants:
-------------------------------------------------------------------*/
const char* const Translator::TRANS_FNAME    = "cobras_";
const char* const Translator::SETTINGS_INDEX = "Translator/Language";


//*******************************************************************
// Translator                                    PRIVATE CONSTRUCTOR
//*******************************************************************
Translator::Translator() : QTranslator()
, current_language_ ( ENGLISH )
{
   setParent( qApp );
   qApp->installTranslator( this );
   load( QString() );
}
// end of Translator

//*******************************************************************
// ~Translator                                            DESTRUCTOR
//*******************************************************************
Translator::~Translator()
{
}
// end of Translator

//*******************************************************************
// instance                                            PUBLIC static
//*******************************************************************
Translator* Translator::instance()
{
   if( 0 == instance_ ) {
      instance_ = new Translator();
   }
   return instance_;
}
// end of instance

//*******************************************************************
// current_lang                                               PUBLIC
//*******************************************************************
Translator::Language Translator::current_lang() const
{
   return current_language_;
}
// end of current_lang

//*******************************************************************
// change_to                                                  PUBLIC
//*******************************************************************
void Translator::change_to( const Language in_language )
{
   const QString fname = get_fname( in_language );
   current_language_ = ( load( fname, qApp->applicationDirPath() ) ) ? in_language : ENGLISH;
   save_settings();
}
// end of change_to

//*******************************************************************
// restore                                                    PUBLIC
//*******************************************************************
void Translator::restore()
{
   read_settings();
   change_to( current_language_ );
}
// end of restore

//*******************************************************************
// get_fname                                                 PRIVATE
//*******************************************************************
QString Translator::get_fname( const Language in_language ) const
{
   QString retval = TRANS_FNAME;
   
   if( POLISH == in_language ) {
      retval += "pl";
   }
   else {
      retval.clear();
   }
   
   return retval;
}
// end of get_fname

//*******************************************************************
// read_settings                                             PRIVATE
//*******************************************************************
void Translator::read_settings()
{
   Settings stt;
   QVariant data;

   if( stt.read( SETTINGS_INDEX, data ) ) {
      current_language_ = static_cast<Language>( data.toInt() );
   }
   else {
      current_language_ = ENGLISH;
   }
}
// end of read_settings

//*******************************************************************
// save_settings                                             PRIVATE
//*******************************************************************
void Translator::save_settings() const
{
   Settings stt;
   stt.save( SETTINGS_INDEX, current_language_ );
}
// end of save_settings
